"""
    domonic.constants.color
    ====================================
"""

from domonic.geom import vec3, vec4


class Color():
    """
        a class for all possible colors
    """

    @staticmethod
    def random_hex():
        """[random_hex]

        Returns:
            [str]: [random hex color]
        """
        import random

        def r():
            return random.randint(0, 255)
        # r = lambda: random.randint(0, 255)
        return '#%02X%02X%02X' % (r(), r(), r())
        # import secrets
        # rgba = '#'+secrets.token_hex(4)

    @staticmethod
    def hex2rgb(h):
        """[hex2rgb]

        Args:
            h ([str]): [hex string]

        Returns:
            [tuple]: [rgb tuple]
        """
        if h[0] == '#':
            h = h.lstrip('#')
        return tuple(int(h[i:i + 2], 16) for i in (0, 2, 4))

    # @staticmethod
    # def hsl2rgb(h, s, l):
    #     """ convert hsl to rgb """
    #     if s == 0:
    #         return l, l, l
    #     if l < 0.5:
    #         if s == 1:
    #             return l * (1 + l), l * 2, l * 2
    #         if s == 2:
    #             return l * 2, l * (1 + l), l * 2
    #         if s == 3:
    #             return l * 2, l * 2, l * (1 + l)
    #     else:
    #         if s == 1:
    #             return l * (1 - l), l * 2, l * 2
    #         if s == 2:
    #             return l * (1 - l), l * 2, l * (1 + l)
    #         if s == 3:
    #             return l * (1 - l), l * (1 + l), l * 2

    # @staticmethod
    # def rgb2hsl(r, g, b):
    #     """ convert rgb to hsl """
    #     maxv = max(r, g, b)
    #     minv = min(r, g, b)
    #     l = (maxv + minv) / 2
    #     if maxv == minv:
    #         return 0.0, 0.0, l
    #     s = (maxv - minv) / (maxv + minv)
    #     if r == maxv:
    #         h = (g - b) / (maxv - minv)
    #     elif g == maxv:
    #         h = 2 + (b - r) / (maxv - minv)
    #     else:
    #         h = 4 + (r - g) / (maxv - minv)
    #     h *= 60
    #     if h < 0:
    #         h += 360
    #     return h, s, l

    @staticmethod
    def rgb2hex(a, b, c):
        """[rgb2hex]

        Args:
            a ([type]): [r]
            b ([type]): [g]
            c ([type]): [b]

        Returns:
            [str]: [retuns a hex string]
        """
        #  TODO - pass tuples or
        # if isinstance(a, (int, float)):
        # elif isinstance(a, (tuple, list)):
        return '#%02x%02x%02x' % (a, b, c)

    @staticmethod
    def fromRGBA(r, g, b, a):
        return Color(r, g, b, a)

    # @staticmethod
    # def fromHsl(h, s, l):
    #     return Color(h, s, l)

    @staticmethod
    def fromHex(hex):
        """ create a color from a hex string """
        return Color(hex)

    def __init__(self, *args, **kwargs):
        # TODO - if type is vec4 / if hash / if word
        if isinstance(args[0], vec4):
            self.r = args[0][0]
            self.g = args[0][1]
            self.b = args[0][2]
            self.a = args[0][3]
        if isinstance(args[0], vec3):
            self.r = args[0][0]
            self.g = args[0][1]
            self.b = args[0][2]
        if isinstance(args[0], str):
            if args[0].startswith('#'):
                self.r, self.g, self.b = Color.hex2rgb(args[0])
        if isinstance(args[0], (int, float)):
            if len(args) == 3:
                self.r, self.g, self.b = args
            if len(args) == 4:
                self.r, self.g, self.b, self.a = args

        self.alpha = kwargs.get('alpha', 1.0)
        self.red = kwargs.get('red', self.r)
        self.green = kwargs.get('green', self.g)
        self.blue = kwargs.get('blue', self.b)
        # self.gray = kwargs.get('gray', self.r)
        self.hue = kwargs.get('hue', 0.0)
        self.saturation = kwargs.get('saturation', 1.0)
        self.brightness = kwargs.get('brightness', 1.0)
        self.lightness = kwargs.get('lightness', 1.0)

        # print(self.r, self.g, self.b)

    def toRGB(self):
        """[returns the color as RGB]

        Returns:
            [tuple]: [ (r, g, b) ]
        """
        return (self.r, self.g, self.b)

    def toHsl(self):
        """ returns the hsl for the color """
        return (self.hue, self.saturation, self.brightness)

    # def toString(self):
        # return str(self)

    def __str__(self):
        return Color.rgb2hex(self.r, self.g, self.b)

    def toHsv(self):
        """ get the hsv for the color """
        return (self.hue, self.saturation, self.brightness)

    def toCSS(self):
        """ return the color as a CSS string """
        return '#%02x%02x%02x' % (self.r, self.g, self.b)

    def toHex(self):
        return str(self)

    def toRGBA(self):
        return (self.r, self.g, self.b, self.a)

    def convert(self, to):
        """ convert the color to another color """
        if to == 'rgb':
            return self.toRGB()
        if to == 'hsl':
            return self.toHsl()
        if to == 'hsv':
            return self.toHsv()
        if to == 'hex':
            return self.toHex()
        if to == 'css':
            return self.toCSS()

    # set(*args)

    def hasAlpha(self):
        """[does the color have an alpha channel]

        Returns:
            [bool]: [True if alpha channel exists else False]
        """
        return self.a > 0

    def equals(self, color):
        return self.r == color.r and self.g == color.g and self.b == color.b

    def __eq__(self, other):
        """ check if two colors are equal """
        return self.r == other.r and self.g == other.g and self.b == other.b

    def __add__(self, other):
        """ add two colors together """
        return Color(self.r + other.r, self.g + other.g, self.b + other.b)

    def __sub__(self, color):
        """ subtract a color from this color """
        return Color(self.r - color.r, self.g - color.g, self.b - color.b)

    def __mul__(self, color):
        """ multiply a color with this color """
        return Color(self.r * color.r, self.g * color.g, self.b * color.b)

    def __div__(self, color):
        """ divide a color with this color """
        return Color(self.r / color.r, self.g / color.g, self.b / color.b)

    # web
    Black = "#000000"  #:
    Navy = "#000080"  #:
    DarkBlue = "#00008B"  #:
    MediumBlue = "#0000CD"  #:
    Blue = "#0000FF"  #:
    DarkGreen = "#006400"  #:
    Green = "#008000"  #:
    Teal = "#008080"  #:
    DarkCyan = "#008B8B"  #:
    DeepSkyBlue = "#00BFFF"  #:
    DarkTurquoise = "#00CED1"  #:
    MediumSpringGreen = "#00FA9A"  #:
    Lime = "#00FF00"  #:
    SpringGreen = "#00FF7F"  #:
    Aqua = "#00FFFF"  #:
    Cyan = "#00FFFF"  #:
    MidnightBlue = "#191970"  #:
    DodgerBlue = "#1E90FF"  #:
    LightSeaGreen = "#20B2AA"  #:
    ForestGreen = "#228B22"  #:
    SeaGreen = "#2E8B57"  #:
    DarkSlateGray = "#2F4F4F"  #:
    DarkSlateGrey = "#2F4F4F"  #:
    LimeGreen = "#32CD32"  #:
    MediumSeaGreen = "#3CB371"  #:
    Turquoise = "#40E0D0"  #:
    RoyalBlue = "#4169E1"  #:
    SteelBlue = "#4682B4"  #:
    DarkSlateBlue = "#483D8B"  #:
    MediumTurquoise = "#48D1CC"  #:
    Indigo = "#4B0082"  #:
    DarkOliveGreen = "#556B2F"  #:
    CadetBlue = "#5F9EA0"  #:
    CornflowerBlue = "#6495ED"  #:
    RebeccaPurple = "#663399"  #:
    MediumAquaMarine = "#66CDAA"  #:
    DimGray = "#696969"  #:
    DimGrey = "#696969"  #:
    SlateBlue = "#6A5ACD"  #:
    OliveDrab = "#6B8E23"  #:
    SlateGray = "#708090"  #:
    SlateGrey = "#708090"  #:
    LightSlateGray = "#778899"  #:
    LightSlateGrey = "#778899"  #:
    MediumSlateBlue = "#7B68EE"  #:
    LawnGreen = "#7CFC00"  #:
    Chartreuse = "#7FFF00"  #:
    Aquamarine = "#7FFFD4"  #:
    Maroon = "#800000"  #:
    Purple = "#800080"  #:
    Olive = "#808000"  #:
    Gray = "#808080"  #:
    Grey = "#808080"  #:
    SkyBlue = "#87CEEB"  #:
    LightSkyBlue = "#87CEFA"  #:
    BlueViolet = "#8A2BE2"  #:
    DarkRed = "#8B0000"  #:
    DarkMagenta = "#8B008B"  #:
    SaddleBrown = "#8B4513"  #:
    DarkSeaGreen = "#8FBC8F"  #:
    LightGreen = "#90EE90"  #:
    MediumPurple = "#9370DB"  #:
    DarkViolet = "#9400D3"  #:
    PaleGreen = "#98FB98"  #:
    DarkOrchid = "#9932CC"  #:
    YellowGreen = "#9ACD32"  #:
    Sienna = "#A0522D"  #:
    Brown = "#A52A2A"  #:
    DarkGray = "#A9A9A9"  #:
    DarkGrey = "#A9A9A9"  #:
    LightBlue = "#ADD8E6"  #:
    GreenYellow = "#ADFF2F"  #:
    PaleTurquoise = "#AFEEEE"  #:
    LightSteelBlue = "#B0C4DE"  #:
    PowderBlue = "#B0E0E6"  #:
    FireBrick = "#B22222"  #:
    DarkGoldenRod = "#B8860B"  #:
    MediumOrchid = "#BA55D3"  #:
    RosyBrown = "#BC8F8F"  #:
    DarkKhaki = "#BDB76B"  #:
    Silver = "#C0C0C0"  #:
    MediumVioletRed = "#C71585"  #:
    IndianRed = "#CD5C5C"  #:
    Peru = "#CD853F"  #:
    Chocolate = "#D2691E"  #:
    Tan = "#D2B48C"  #:
    LightGray = "#D3D3D3"  #:
    LightGrey = "#D3D3D3"  #:
    Thistle = "#D8BFD8"  #:
    Orchid = "#DA70D6"  #:
    GoldenRod = "#DAA520"  #:
    PaleVioletRed = "#DB7093"  #:
    Crimson = "#DC143C"  #:
    Gainsboro = "#DCDCDC"  #:
    Plum = "#DDA0DD"  #:
    BurlyWood = "#DEB887"  #:
    LightCyan = "#E0FFFF"  #:
    Lavender = "#E6E6FA"  #:
    DarkSalmon = "#E9967A"  #:
    Violet = "#EE82EE"  #:
    PaleGoldenRod = "#EEE8AA"  #:
    LightCoral = "#F08080"  #:
    Khaki = "#F0E68C"  #:
    AliceBlue = "#F0F8FF"  #:
    HoneyDew = "#F0FFF0"  #:
    Azure = "#F0FFFF"  #:
    SandyBrown = "#F4A460"  #:
    Wheat = "#F5DEB3"  #:
    Beige = "#F5F5DC"  #:
    WhiteSmoke = "#F5F5F5"  #:
    MintCream = "#F5FFFA"  #:
    GhostWhite = "#F8F8FF"  #:
    Salmon = "#FA8072"  #:
    AntiqueWhite = "#FAEBD7"  #:
    Linen = "#FAF0E6"  #:
    LightGoldenRodYellow = "#FAFAD2"  #:
    OldLace = "#FDF5E6"  #:
    Red = "#FF0000"  #:
    Fuchsia = "#FF00FF"  #:
    Magenta = "#FF00FF"  #:
    DeepPink = "#FF1493"  #:
    OrangeRed = "#FF4500"  #:
    Tomato = "#FF6347"  #:
    HotPink = "#FF69B4"  #:
    Coral = "#FF7F50"  #:
    DarkOrange = "#FF8C00"  #:
    LightSalmon = "#FFA07A"  #:
    Orange = "#FFA500"  #:
    LightPink = "#FFB6C1"  #:
    Pink = "#FFC0CB"  #:
    Gold = "#FFD700"  #:
    PeachPuff = "#FFDAB9"  #:
    NavajoWhite = "#FFDEAD"  #:
    Moccasin = "#FFE4B5"  #:
    Bisque = "#FFE4C4"  #:
    MistyRose = "#FFE4E1"  #:
    BlanchedAlmond = "#FFEBCD"  #:
    PapayaWhip = "#FFEFD5"  #:
    LavenderBlush = "#FFF0F5"  #:
    SeaShell = "#FFF5EE"  #:
    Cornsilk = "#FFF8DC"  #:
    LemonChiffon = "#FFFACD"  #:
    FloralWhite = "#FFFAF0"  #:
    Snow = "#FFFAFA"  #:
    Yellow = "#FFFF00"  #:
    LightYellow = "#FFFFE0"  #:
    Ivory = "#FFFFF0"  #:
    White = "#FFFFFF"  #:

    # XKCD
    acidgreen = "#8ffe09"  #:
    adobe = "#bd6c48"  #:
    algae = "#54ac68"  #:
    algaegreen = "#21c36f"  #:
    almostblack = "#070d0d"  #:
    amber = "#feb308"  #:
    amethyst = "#9b5fc0"  #:
    apple = "#6ecb3c"  #:
    applegreen = "#76cd26"  #:
    apricot = "#ffb16d"  #:
    aqua = "#13eac9"  #:
    aquablue = "#02d8e9"  #:
    aquagreen = "#12e193"  #:
    aquamarine = "#2ee8bb"  #:
    armygreen = "#4b5d16"  #:
    asparagus = "#77ab56"  #:
    aubergine = "#3d0734"  #:
    auburn = "#9a3001"  #:
    avocado = "#90b134"  #:
    avocadogreen = "#87a922"  #:
    azul = "#1d5dec"  #:
    azure = "#069af3"  #:
    babyblue = "#a2cffe"  #:
    babygreen = "#8cff9e"  #:
    babypink = "#ffb7ce"  #:
    babypoo = "#ab9004"  #:
    babypoop = "#937c00"  #:
    babypoopgreen = "#8f9805"  #:
    babypukegreen = "#b6c406"  #:
    babypurple = "#ca9bf7"  #:
    babyshitbrown = "#ad900d"  #:
    babyshitgreen = "#889717"  #:
    banana = "#ffff7e"  #:
    bananayellow = "#fafe4b"  #:
    barbiepink = "#fe46a5"  #:
    barfgreen = "#94ac02"  #:
    barney = "#ac1db8"  #:
    barneypurple = "#a00498"  #:
    battleshipgrey = "#6b7c85"  #:
    beige = "#e6daa6"  #:
    berry = "#990f4b"  #:
    bile = "#b5c306"  #:
    black = "#000000"  #:
    bland = "#afa88b"  #:
    blood = "#770001"  #:
    bloodorange = "#fe4b03"  #:
    bloodred = "#980002"  #:
    blue = "#0343df"  #:
    blueberry = "#464196"  #:
    blueblue = "#2242c7"  #:
    bluegreen = "#0f9b8e"  #:
    bluegrey = "#85a3b2"  #:
    bluepurple = "#5a06ef"  #:
    blueviolet = "#5d06e9"  #:
    bluewithahintofpurple = "#533cc6"  #:
    blueygreen = "#2bb179"  #:
    blueygrey = "#89a0b0"  #:
    blueypurple = "#6241c7"  #:
    bluish = "#2976bb"  #:
    bluishgreen = "#10a674"  #:
    bluishgrey = "#748b97"  #:
    bluishpurple = "#703be7"  #:
    blurple = "#5539cc"  #:
    blush = "#f29e8e"  #:
    blushpink = "#fe828c"  #:
    booger = "#9bb53c"  #:
    boogergreen = "#96b403"  #:
    bordeaux = "#7b002c"  #:
    boringgreen = "#63b365"  #:
    bottlegreen = "#044a05"  #:
    brick = "#a03623"  #:
    brickorange = "#c14a09"  #:
    brickred = "#8f1402"  #:
    brightaqua = "#0bf9ea"  #:
    brightblue = "#0165fc"  #:
    brightcyan = "#41fdfe"  #:
    brightgreen = "#01ff07"  #:
    brightlavender = "#c760ff"  #:
    brightlightblue = "#26f7fd"  #:
    brightlightgreen = "#2dfe54"  #:
    brightlilac = "#c95efb"  #:
    brightlime = "#87fd05"  #:
    brightlimegreen = "#65fe08"  #:
    brightmagenta = "#ff08e8"  #:
    brightolive = "#9cbb04"  #:
    brightorange = "#ff5b00"  #:
    brightpink = "#fe01b1"  #:
    brightpurple = "#be03fd"  #:
    brightred = "#ff000d"  #:
    brightseagreen = "#05ffa6"  #:
    brightskyblue = "#02ccfe"  #:
    brightteal = "#01f9c6"  #:
    brightturquoise = "#0ffef9"  #:
    brightviolet = "#ad0afd"  #:
    brightyellow = "#fffd01"  #:
    brightyellowgreen = "#9dff00"  #:
    britishracinggreen = "#05480d"  #:
    bronze = "#a87900"  #:
    brown = "#653700"  #:
    browngreen = "#706c11"  #:
    browngrey = "#8d8468"  #:
    brownish = "#9c6d57"  #:
    brownishgreen = "#6a6e09"  #:
    brownishgrey = "#86775f"  #:
    brownishorange = "#cb7723"  #:
    brownishpink = "#c27e79"  #:
    brownishpurple = "#76424e"  #:
    brownishred = "#9e3623"  #:
    brownishyellow = "#c9b003"  #:
    brownorange = "#b96902"  #:
    brownred = "#922b05"  #:
    brownyellow = "#b29705"  #:
    brownygreen = "#6f6c0a"  #:
    brownyorange = "#ca6b02"  #:
    bruise = "#7e4071"  #:
    bubblegum = "#ff6cb5"  #:
    bubblegumpink = "#ff69af"  #:
    buff = "#fef69e"  #:
    burgundy = "#610023"  #:
    burntorange = "#c04e01"  #:
    burntred = "#9f2305"  #:
    burntsiena = "#b75203"  #:
    burntsienna = "#b04e0f"  #:
    burntumber = "#a0450e"  #:
    burntyellow = "#d5ab09"  #:
    burple = "#6832e3"  #:
    butter = "#ffff81"  #:
    butterscotch = "#fdb147"  #:
    butteryellow = "#fffd74"  #:
    cadetblue = "#4e7496"  #:
    camel = "#c69f59"  #:
    camo = "#7f8f4e"  #:
    camogreen = "#526525"  #:
    camouflagegreen = "#4b6113"  #:
    canary = "#fdff63"  #:
    canaryyellow = "#fffe40"  #:
    candypink = "#ff63e9"  #:
    caramel = "#af6f09"  #:
    carmine = "#9d0216"  #:
    carnation = "#fd798f"  #:
    carnationpink = "#ff7fa7"  #:
    carolinablue = "#8ab8fe"  #:
    celadon = "#befdb7"  #:
    celery = "#c1fd95"  #:
    cement = "#a5a391"  #:
    cerise = "#de0c62"  #:
    cerulean = "#0485d1"  #:
    ceruleanblue = "#056eee"  #:
    charcoal = "#343837"  #:
    charcoalgrey = "#3c4142"  #:
    chartreuse = "#c1f80a"  #:
    cherry = "#cf0234"  #:
    cherryred = "#f7022a"  #:
    chestnut = "#742802"  #:
    chocolate = "#3d1c02"  #:
    chocolatebrown = "#411900"  #:
    cinnamon = "#ac4f06"  #:
    claret = "#680018"  #:
    clay = "#b66a50"  #:
    claybrown = "#b2713d"  #:
    clearblue = "#247afd"  #:
    cobalt = "#1e488f"  #:
    cobaltblue = "#030aa7"  #:
    cocoa = "#875f42"  #:
    coffee = "#a6814c"  #:
    coolblue = "#4984b8"  #:
    coolgreen = "#33b864"  #:
    coolgrey = "#95a3a6"  #:
    copper = "#b66325"  #:
    coral = "#fc5a50"  #:
    coralpink = "#ff6163"  #:
    cornflower = "#6a79f7"  #:
    cornflowerblue = "#5170d7"  #:
    cranberry = "#9e003a"  #:
    cream = "#ffffc2"  #:
    creme = "#ffffb6"  #:
    crimson = "#8c000f"  #:
    custard = "#fffd78"  #:
    cyan = "#00ffff"  #:
    dandelion = "#fedf08"  #:
    dark = "#1b2431"  #:
    darkaqua = "#05696b"  #:
    darkaquamarine = "#017371"  #:
    darkbeige = "#ac9362"  #:
    darkblue = "#030764"  #:
    darkbluegreen = "#005249"  #:
    darkbluegrey = "#1f3b4d"  #:
    darkbrown = "#341c02"  #:
    darkcoral = "#cf524e"  #:
    darkcream = "#fff39a"  #:
    darkcyan = "#0a888a"  #:
    darkforestgreen = "#002d04"  #:
    darkfuchsia = "#9d0759"  #:
    darkgold = "#b59410"  #:
    darkgrassgreen = "#388004"  #:
    darkgreen = "#054907"  #:
    darkgreenblue = "#1f6357"  #:
    darkgrey = "#363737"  #:
    darkgreyblue = "#29465b"  #:
    darkhotpink = "#d90166"  #:
    darkindigo = "#1f0954"  #:
    darkishblue = "#014182"  #:
    darkishgreen = "#287c37"  #:
    darkishpink = "#da467d"  #:
    darkishpurple = "#751973"  #:
    darkishred = "#a90308"  #:
    darkkhaki = "#9b8f55"  #:
    darklavender = "#856798"  #:
    darklilac = "#9c6da5"  #:
    darklime = "#84b701"  #:
    darklimegreen = "#7ebd01"  #:
    darkmagenta = "#960056"  #:
    darkmaroon = "#3c0008"  #:
    darkmauve = "#874c62"  #:
    darkmint = "#48c072"  #:
    darkmintgreen = "#20c073"  #:
    darkmustard = "#a88905"  #:
    darknavy = "#000435"  #:
    darknavyblue = "#00022e"  #:
    darkolive = "#373e02"  #:
    darkolivegreen = "#3c4d03"  #:
    darkorange = "#c65102"  #:
    darkpastelgreen = "#56ae57"  #:
    darkpeach = "#de7e5d"  #:
    darkperiwinkle = "#665fd1"  #:
    darkpink = "#cb416b"  #:
    darkplum = "#3f012c"  #:
    darkpurple = "#35063e"  #:
    darkred = "#840000"  #:
    darkrose = "#b5485d"  #:
    darkroyalblue = "#02066f"  #:
    darksage = "#598556"  #:
    darksalmon = "#c85a53"  #:
    darksand = "#a88f59"  #:
    darkseafoam = "#1fb57a"  #:
    darkseafoamgreen = "#3eaf76"  #:
    darkseagreen = "#11875d"  #:
    darkskyblue = "#448ee4"  #:
    darkslateblue = "#214761"  #:
    darktan = "#af884a"  #:
    darktaupe = "#7f684e"  #:
    darkteal = "#014d4e"  #:
    darkturquoise = "#045c5a"  #:
    darkviolet = "#34013f"  #:
    darkyellow = "#d5b60a"  #:
    darkyellowgreen = "#728f02"  #:
    deepaqua = "#08787f"  #:
    deepblue = "#040273"  #:
    deepbrown = "#410200"  #:
    deepgreen = "#02590f"  #:
    deeplavender = "#8d5eb7"  #:
    deeplilac = "#966ebd"  #:
    deepmagenta = "#a0025c"  #:
    deeporange = "#dc4d01"  #:
    deeppink = "#cb0162"  #:
    deeppurple = "#36013f"  #:
    deepred = "#9a0200"  #:
    deeprose = "#c74767"  #:
    deepseablue = "#015482"  #:
    deepskyblue = "#0d75f8"  #:
    deepteal = "#00555a"  #:
    deepturquoise = "#017374"  #:
    deepviolet = "#490648"  #:
    denim = "#3b638c"  #:
    denimblue = "#3b5b92"  #:
    desert = "#ccad60"  #:
    diarrhea = "#9f8303"  #:
    dirt = "#8a6e45"  #:
    dirtbrown = "#836539"  #:
    dirtyblue = "#3f829d"  #:
    dirtygreen = "#667e2c"  #:
    dirtyorange = "#c87606"  #:
    dirtypink = "#ca7b80"  #:
    dirtypurple = "#734a65"  #:
    dirtyyellow = "#cdc50a"  #:
    dodgerblue = "#3e82fc"  #:
    drab = "#828344"  #:
    drabgreen = "#749551"  #:
    driedblood = "#4b0101"  #:
    duckeggblue = "#c3fbf4"  #:
    dullblue = "#49759c"  #:
    dullbrown = "#876e4b"  #:
    dullgreen = "#74a662"  #:
    dullorange = "#d8863b"  #:
    dullpink = "#d5869d"  #:
    dullpurple = "#84597e"  #:
    dullred = "#bb3f3f"  #:
    dullteal = "#5f9e8f"  #:
    dullyellow = "#eedc5b"  #:
    dusk = "#4e5481"  #:
    duskblue = "#26538d"  #:
    duskyblue = "#475f94"  #:
    duskypink = "#cc7a8b"  #:
    duskypurple = "#895b7b"  #:
    duskyrose = "#ba6873"  #:
    dust = "#b2996e"  #:
    dustyblue = "#5a86ad"  #:
    dustygreen = "#76a973"  #:
    dustylavender = "#ac86a8"  #:
    dustyorange = "#f0833a"  #:
    dustypink = "#d58a94"  #:
    dustypurple = "#825f87"  #:
    dustyred = "#b9484e"  #:
    dustyrose = "#c0737a"  #:
    dustyteal = "#4c9085"  #:
    earth = "#a2653e"  #:
    eastergreen = "#8cfd7e"  #:
    easterpurple = "#c071fe"  #:
    ecru = "#feffca"  #:
    eggplant = "#380835"  #:
    eggplantpurple = "#430541"  #:
    eggshell = "#fffcc4"  #:
    eggshellblue = "#c4fff7"  #:
    electricblue = "#0652ff"  #:
    electricgreen = "#21fc0d"  #:
    electriclime = "#a8ff04"  #:
    electricpink = "#ff0490"  #:
    electricpurple = "#aa23ff"  #:
    emerald = "#01a049"  #:
    emeraldgreen = "#028f1e"  #:
    evergreen = "#05472a"  #:
    fadedblue = "#658cbb"  #:
    fadedgreen = "#7bb274"  #:
    fadedorange = "#f0944d"  #:
    fadedpink = "#de9dac"  #:
    fadedpurple = "#916e99"  #:
    fadedred = "#d3494e"  #:
    fadedyellow = "#feff7f"  #:
    fawn = "#cfaf7b"  #:
    fern = "#63a950"  #:
    ferngreen = "#548d44"  #:
    fireenginered = "#fe0002"  #:
    flatblue = "#3c73a8"  #:
    flatgreen = "#699d4c"  #:
    fluorescentgreen = "#08ff08"  #:
    flurogreen = "#0aff02"  #:
    foamgreen = "#90fda9"  #:
    forest = "#0b5509"  #:
    forestgreen = "#06470c"  #:
    forrestgreen = "#154406"  #:
    frenchblue = "#436bad"  #:
    freshgreen = "#69d84f"  #:
    froggreen = "#58bc08"  #:
    fuchsia = "#ed0dd9"  #:
    gold = "#dbb40c"  #:
    golden = "#f5bf03"  #:
    goldenbrown = "#b27a01"  #:
    goldenrod = "#f9bc08"  #:
    goldenyellow = "#fec615"  #:
    grape = "#6c3461"  #:
    grapefruit = "#fd5956"  #:
    grapepurple = "#5d1451"  #:
    grass = "#5cac2d"  #:
    grassgreen = "#3f9b0b"  #:
    grassygreen = "#419c03"  #:
    green = "#15b01a"  #:
    greenapple = "#5edc1f"  #:
    greenblue = "#01c08d"  #:
    greenbrown = "#544e03"  #:
    greengrey = "#77926f"  #:
    greenish = "#40a368"  #:
    greenishbeige = "#c9d179"  #:
    greenishblue = "#0b8b87"  #:
    greenishbrown = "#696112"  #:
    greenishcyan = "#2afeb7"  #:
    greenishgrey = "#96ae8d"  #:
    greenishtan = "#bccb7a"  #:
    greenishteal = "#32bf84"  #:
    greenishturquoise = "#00fbb0"  #:
    greenishyellow = "#cdfd02"  #:
    greenteal = "#0cb577"  #:
    greenyblue = "#42b395"  #:
    greenybrown = "#696006"  #:
    greenyellow = "#b5ce08"  #:
    greenygrey = "#7ea07a"  #:
    greenyyellow = "#c6f808"  #:
    grey = "#929591"  #:
    greyblue = "#647d8e"  #:
    greybrown = "#7f7053"  #:
    greygreen = "#86a17d"  #:
    greyish = "#a8a495"  #:
    greyishblue = "#5e819d"  #:
    greyishbrown = "#7a6a4f"  #:
    greyishgreen = "#82a67d"  #:
    greyishpink = "#c88d94"  #:
    greyishpurple = "#887191"  #:
    greyishteal = "#719f91"  #:
    greypink = "#c3909b"  #:
    greypurple = "#826d8c"  #:
    greyteal = "#5e9b8a"  #:
    grossgreen = "#a0bf16"  #:
    gunmetal = "#536267"  #:
    hazel = "#8e7618"  #:
    heather = "#a484ac"  #:
    heliotrope = "#d94ff5"  #:
    highlightergreen = "#1bfc06"  #:
    hospitalgreen = "#9be5aa"  #:
    hotgreen = "#25ff29"  #:
    hotmagenta = "#f504c9"  #:
    hotpink = "#ff028d"  #:
    hotpurple = "#cb00f5"  #:
    huntergreen = "#0b4008"  #:
    ice = "#d6fffa"  #:
    iceblue = "#d7fffe"  #:
    ickygreen = "#8fae22"  #:
    indianred = "#850e04"  #:
    indigo = "#380282"  #:
    indigoblue = "#3a18b1"  #:
    iris = "#6258c4"  #:
    irishgreen = "#019529"  #:
    ivory = "#ffffcb"  #:
    jade = "#1fa774"  #:
    jadegreen = "#2baf6a"  #:
    junglegreen = "#048243"  #:
    kelleygreen = "#009337"  #:
    kellygreen = "#02ab2e"  #:
    kermitgreen = "#5cb200"  #:
    keylime = "#aeff6e"  #:
    khaki = "#aaa662"  #:
    khakigreen = "#728639"  #:
    kiwi = "#9cef43"  #:
    kiwigreen = "#8ee53f"  #:
    lavender = "#c79fef"  #:
    lavenderblue = "#8b88f8"  #:
    lavenderpink = "#dd85d7"  #:
    lawngreen = "#4da409"  #:
    leaf = "#71aa34"  #:
    leafgreen = "#5ca904"  #:
    leafygreen = "#51b73b"  #:
    leather = "#ac7434"  #:
    lemon = "#fdff52"  #:
    lemongreen = "#adf802"  #:
    lemonlime = "#bffe28"  #:
    lemonyellow = "#fdff38"  #:
    lichen = "#8fb67b"  #:
    lightaqua = "#8cffdb"  #:
    lightaquamarine = "#7bfdc7"  #:
    lightbeige = "#fffeb6"  #:
    lightblue = "#7bc8f6"  #:
    lightbluegreen = "#7efbb3"  #:
    lightbluegrey = "#b7c9e2"  #:
    lightbluishgreen = "#76fda8"  #:
    lightbrightgreen = "#53fe5c"  #:
    lightbrown = "#ad8150"  #:
    lightburgundy = "#a8415b"  #:
    lightcyan = "#acfffc"  #:
    lighteggplant = "#894585"  #:
    lightergreen = "#75fd63"  #:
    lighterpurple = "#a55af4"  #:
    lightforestgreen = "#4f9153"  #:
    lightgold = "#fddc5c"  #:
    lightgrassgreen = "#9af764"  #:
    lightgreen = "#76ff7b"  #:
    lightgreenblue = "#56fca2"  #:
    lightgreenishblue = "#63f7b4"  #:
    lightgrey = "#d8dcd6"  #:
    lightgreyblue = "#9dbcd4"  #:
    lightgreygreen = "#b7e1a1"  #:
    lightindigo = "#6d5acf"  #:
    lightishblue = "#3d7afd"  #:
    lightishgreen = "#61e160"  #:
    lightishpurple = "#a552e6"  #:
    lightishred = "#fe2f4a"  #:
    lightkhaki = "#e6f2a2"  #:
    lightlavendar = "#efc0fe"  #:
    lightlavender = "#dfc5fe"  #:
    lightlightblue = "#cafffb"  #:
    lightlightgreen = "#c8ffb0"  #:
    lightlilac = "#edc8ff"  #:
    lightlime = "#aefd6c"  #:
    lightlimegreen = "#b9ff66"  #:
    lightmagenta = "#fa5ff7"  #:
    lightmaroon = "#a24857"  #:
    lightmauve = "#c292a1"  #:
    lightmint = "#b6ffbb"  #:
    lightmintgreen = "#a6fbb2"  #:
    lightmossgreen = "#a6c875"  #:
    lightmustard = "#f7d560"  #:
    lightnavy = "#155084"  #:
    lightnavyblue = "#2e5a88"  #:
    lightneongreen = "#4efd54"  #:
    lightolive = "#acbf69"  #:
    lightolivegreen = "#a4be5c"  #:
    lightorange = "#fdaa48"  #:
    lightpastelgreen = "#b2fba5"  #:
    lightpeach = "#ffd8b1"  #:
    lightpeagreen = "#c4fe82"  #:
    lightperiwinkle = "#c1c6fc"  #:
    lightpink = "#ffd1df"  #:
    lightplum = "#9d5783"  #:
    lightpurple = "#bf77f6"  #:
    lightred = "#ff474c"  #:
    lightrose = "#ffc5cb"  #:
    lightroyalblue = "#3a2efe"  #:
    lightsage = "#bcecac"  #:
    lightsalmon = "#fea993"  #:
    lightseafoam = "#a0febf"  #:
    lightseafoamgreen = "#a7ffb5"  #:
    lightseagreen = "#98f6b0"  #:
    lightskyblue = "#c6fcff"  #:
    lighttan = "#fbeeac"  #:
    lightteal = "#90e4c1"  #:
    lightturquoise = "#7ef4cc"  #:
    lighturple = "#b36ff6"  #:
    lightviolet = "#d6b4fc"  #:
    lightyellow = "#fffe7a"  #:
    lightyellowgreen = "#ccfd7f"  #:
    lightyellowishgreen = "#c2ff89"  #:
    lilac = "#cea2fd"  #:
    liliac = "#c48efd"  #:
    lime = "#aaff32"  #:
    limegreen = "#89fe05"  #:
    limeyellow = "#d0fe1d"  #:
    lipstick = "#d5174e"  #:
    lipstickred = "#c0022f"  #:
    macaroniandcheese = "#efb435"  #:
    magenta = "#c20078"  #:
    mahogany = "#4a0100"  #:
    maize = "#f4d054"  #:
    mango = "#ffa62b"  #:
    manilla = "#fffa86"  #:
    marigold = "#fcc006"  #:
    marine = "#042e60"  #:
    marineblue = "#01386a"  #:
    maroon = "#650021"  #:
    mauve = "#ae7181"  #:
    mediumblue = "#2c6fbb"  #:
    mediumbrown = "#7f5112"  #:
    mediumgreen = "#39ad48"  #:
    mediumgrey = "#7d7f7c"  #:
    mediumpink = "#f36196"  #:
    mediumpurple = "#9e43a2"  #:
    melon = "#ff7855"  #:
    merlot = "#730039"  #:
    metallicblue = "#4f738e"  #:
    midblue = "#276ab3"  #:
    midgreen = "#50a747"  #:
    midnight = "#03012d"  #:
    midnightblue = "#020035"  #:
    midnightpurple = "#280137"  #:
    militarygreen = "#667c3e"  #:
    milkchocolate = "#7f4e1e"  #:
    mint = "#9ffeb0"  #:
    mintgreen = "#8fff9f"  #:
    mintygreen = "#0bf77d"  #:
    mocha = "#9d7651"  #:
    moss = "#769958"  #:
    mossgreen = "#658b38"  #:
    mossygreen = "#638b27"  #:
    mud = "#735c12"  #:
    mudbrown = "#60460f"  #:
    muddybrown = "#886806"  #:
    muddygreen = "#657432"  #:
    muddyyellow = "#bfac05"  #:
    mudgreen = "#606602"  #:
    mulberry = "#920a4e"  #:
    murkygreen = "#6c7a0e"  #:
    mushroom = "#ba9e88"  #:
    mustard = "#ceb301"  #:
    mustardbrown = "#ac7e04"  #:
    mustardgreen = "#a8b504"  #:
    mustardyellow = "#d2bd0a"  #:
    mutedblue = "#3b719f"  #:
    mutedgreen = "#5fa052"  #:
    mutedpink = "#d1768f"  #:
    mutedpurple = "#805b87"  #:
    nastygreen = "#70b23f"  #:
    navy = "#01153e"  #:
    navyblue = "#001146"  #:
    navygreen = "#35530a"  #:
    neonblue = "#04d9ff"  #:
    neongreen = "#0cff0c"  #:
    neonpink = "#fe019a"  #:
    neonpurple = "#bc13fe"  #:
    neonred = "#ff073a"  #:
    neonyellow = "#cfff04"  #:
    niceblue = "#107ab0"  #:
    nightblue = "#040348"  #:
    ocean = "#017b92"  #:
    oceanblue = "#03719c"  #:
    oceangreen = "#3d9973"  #:
    ocher = "#bf9b0c"  #:
    ochre = "#bf9005"  #:
    ocre = "#c69c04"  #:
    offblue = "#5684ae"  #:
    offgreen = "#6ba353"  #:
    offwhite = "#ffffe4"  #:
    offyellow = "#f1f33f"  #:
    oldpink = "#c77986"  #:
    oldrose = "#c87f89"  #:
    olive = "#6e750e"  #:
    olivebrown = "#645403"  #:
    olivedrab = "#6f7632"  #:
    olivegreen = "#677a04"  #:
    oliveyellow = "#c2b709"  #:
    orange = "#f97306"  #:
    orangebrown = "#be6400"  #:
    orangeish = "#fd8d49"  #:
    orangepink = "#ff6f52"  #:
    orangered = "#fe420f"  #:
    orangeybrown = "#b16002"  #:
    orangeyellow = "#ffad01"  #:
    orangeyred = "#fa4224"  #:
    orangeyyellow = "#fdb915"  #:
    orangish = "#fc824a"  #:
    orangishbrown = "#b25f03"  #:
    orangishred = "#f43605"  #:
    orchid = "#c875c4"  #:
    pale = "#fff9d0"  #:
    paleaqua = "#b8ffeb"  #:
    paleblue = "#d0fefe"  #:
    palebrown = "#b1916e"  #:
    palecyan = "#b7fffa"  #:
    palegold = "#fdde6c"  #:
    palegreen = "#c7fdb5"  #:
    palegrey = "#fdfdfe"  #:
    palelavender = "#eecffe"  #:
    palelightgreen = "#b1fc99"  #:
    palelilac = "#e4cbff"  #:
    palelime = "#befd73"  #:
    palelimegreen = "#b1ff65"  #:
    palemagenta = "#d767ad"  #:
    palemauve = "#fed0fc"  #:
    paleolive = "#b9cc81"  #:
    paleolivegreen = "#b1d27b"  #:
    paleorange = "#ffa756"  #:
    palepeach = "#ffe5ad"  #:
    palepink = "#ffcfdc"  #:
    palepurple = "#b790d4"  #:
    palered = "#d9544d"  #:
    palerose = "#fdc1c5"  #:
    palesalmon = "#ffb19a"  #:
    paleskyblue = "#bdf6fe"  #:
    paleteal = "#82cbb2"  #:
    paleturquoise = "#a5fbd5"  #:
    paleviolet = "#ceaefa"  #:
    paleyellow = "#ffff84"  #:
    parchment = "#fefcaf"  #:
    pastelblue = "#a2bffe"  #:
    pastelgreen = "#b0ff9d"  #:
    pastelorange = "#ff964f"  #:
    pastelpink = "#ffbacd"  #:
    pastelpurple = "#caa0ff"  #:
    pastelred = "#db5856"  #:
    pastelyellow = "#fffe71"  #:
    pea = "#a4bf20"  #:
    peach = "#ffb07c"  #:
    peachypink = "#ff9a8a"  #:
    peacockblue = "#016795"  #:
    peagreen = "#8eab12"  #:
    pear = "#cbf85f"  #:
    peasoup = "#929901"  #:
    peasoupgreen = "#94a617"  #:
    periwinkle = "#8e82fe"  #:
    periwinkleblue = "#8f99fb"  #:
    perrywinkle = "#8f8ce7"  #:
    petrol = "#005f6a"  #:
    pigpink = "#e78ea5"  #:
    pine = "#2b5d34"  #:
    pinegreen = "#0a481e"  #:
    pink = "#ff81c0"  #:
    pinkish = "#d46a7e"  #:
    pinkishbrown = "#b17261"  #:
    pinkishgrey = "#c8aca9"  #:
    pinkishorange = "#ff724c"  #:
    pinkishpurple = "#d648d7"  #:
    pinkishred = "#f10c45"  #:
    pinkishtan = "#d99b82"  #:
    pinkpurple = "#ef1de7"  #:
    pinkred = "#f5054f"  #:
    pinky = "#fc86aa"  #:
    pinkypurple = "#c94cbe"  #:
    pinkyred = "#fc2647"  #:
    pissyellow = "#ddd618"  #:
    pistachio = "#c0fa8b"  #:
    plum = "#580f41"  #:
    plumpurple = "#4e0550"  #:
    poisongreen = "#40fd14"  #:
    poo = "#8f7303"  #:
    poobrown = "#885f01"  #:
    poop = "#7f5e00"  #:
    poopbrown = "#7a5901"  #:
    poopgreen = "#6f7c00"  #:
    powderblue = "#b1d1fc"  #:
    powderpink = "#ffb2d0"  #:
    primaryblue = "#0804f9"  #:
    prussianblue = "#004577"  #:
    puce = "#a57e52"  #:
    puke = "#a5a502"  #:
    pukebrown = "#947706"  #:
    pukegreen = "#9aae07"  #:
    pukeyellow = "#c2be0e"  #:
    pumpkin = "#e17701"  #:
    pumpkinorange = "#fb7d07"  #:
    pureblue = "#0203e2"  #:
    purple = "#7e1e9c"  #:
    purpleblue = "#5d21d0"  #:
    purplebrown = "#673a3f"  #:
    purplegrey = "#866f85"  #:
    purpleish = "#98568d"  #:
    purpleishblue = "#6140ef"  #:
    purpleishpink = "#df4ec8"  #:
    purplepink = "#d725de"  #:
    purplered = "#990147"  #:
    purpley = "#8756e4"  #:
    purpleyblue = "#5f34e7"  #:
    purpleygrey = "#947e94"  #:
    purpleypink = "#c83cb9"  #:
    purplish = "#94568c"  #:
    purplishblue = "#601ef9"  #:
    purplishbrown = "#6b4247"  #:
    purplishgrey = "#7a687f"  #:
    purplishpink = "#ce5dae"  #:
    purplishred = "#b0054b"  #:
    purply = "#983fb2"  #:
    purplyblue = "#661aee"  #:
    purplypink = "#f075e6"  #:
    putty = "#beae8a"  #:
    racinggreen = "#014600"  #:
    radioactivegreen = "#2cfa1f"  #:
    raspberry = "#b00149"  #:
    rawsienna = "#9a6200"  #:
    rawumber = "#a75e09"  #:
    reallylightblue = "#d4ffff"  #:
    red = "#e50000"  #:
    redbrown = "#8b2e16"  #:
    reddish = "#c44240"  #:
    reddishbrown = "#7f2b0a"  #:
    reddishgrey = "#997570"  #:
    reddishorange = "#f8481c"  #:
    reddishpink = "#fe2c54"  #:
    reddishpurple = "#910951"  #:
    reddybrown = "#6e1005"  #:
    redorange = "#fd3c06"  #:
    redpink = "#fa2a55"  #:
    redpurple = "#820747"  #:
    redviolet = "#9e0168"  #:
    redwine = "#8c0034"  #:
    richblue = "#021bf9"  #:
    richpurple = "#720058"  #:
    robineggblue = "#8af1fe"  #:
    robinsegg = "#6dedfd"  #:
    robinseggblue = "#98eff9"  #:
    rosa = "#fe86a4"  #:
    rose = "#cf6275"  #:
    rosepink = "#f7879a"  #:
    rosered = "#be013c"  #:
    rosypink = "#f6688e"  #:
    rouge = "#ab1239"  #:
    royal = "#0c1793"  #:
    royalblue = "#0504aa"  #:
    royalpurple = "#4b006e"  #:
    ruby = "#ca0147"  #:
    russet = "#a13905"  #:
    rust = "#a83c09"  #:
    rustbrown = "#8b3103"  #:
    rustorange = "#c45508"  #:
    rustred = "#aa2704"  #:
    rustyorange = "#cd5909"  #:
    rustyred = "#af2f0d"  #:
    saffron = "#feb209"  #:
    sage = "#87ae73"  #:
    sagegreen = "#88b378"  #:
    salmon = "#ff796c"  #:
    salmonpink = "#fe7b7c"  #:
    sand = "#e2ca76"  #:
    sandbrown = "#cba560"  #:
    sandstone = "#c9ae74"  #:
    sandy = "#f1da7a"  #:
    sandybrown = "#c4a661"  #:
    sandyellow = "#fce166"  #:
    sandyyellow = "#fdee73"  #:
    sapgreen = "#5c8b15"  #:
    sapphire = "#2138ab"  #:
    scarlet = "#be0119"  #:
    sea = "#3c9992"  #:
    seablue = "#047495"  #:
    seafoam = "#80f9ad"  #:
    seafoamblue = "#78d1b6"  #:
    seafoamgreen = "#7af9ab"  #:
    seagreen = "#53fca1"  #:
    seaweed = "#18d17b"  #:
    seaweedgreen = "#35ad6b"  #:
    sepia = "#985e2b"  #:
    shamrock = "#01b44c"  #:
    shamrockgreen = "#02c14d"  #:
    shit = "#7f5f00"  #:
    shitbrown = "#7b5804"  #:
    shitgreen = "#758000"  #:
    shockingpink = "#fe02a2"  #:
    sickgreen = "#9db92c"  #:
    sicklygreen = "#94b21c"  #:
    sicklyyellow = "#d0e429"  #:
    sienna = "#a9561e"  #:
    silver = "#c5c9c7"  #:
    sky = "#82cafc"  #:
    skyblue = "#75bbfd"  #:
    slate = "#516572"  #:
    slateblue = "#5b7c99"  #:
    slategreen = "#658d6d"  #:
    slategrey = "#59656d"  #:
    slimegreen = "#99cc04"  #:
    snot = "#acbb0d"  #:
    snotgreen = "#9dc100"  #:
    softblue = "#6488ea"  #:
    softgreen = "#6fc276"  #:
    softpink = "#fdb0c0"  #:
    softpurple = "#a66fb5"  #:
    spearmint = "#1ef876"  #:
    springgreen = "#a9f971"  #:
    spruce = "#0a5f38"  #:
    squash = "#f2ab15"  #:
    steel = "#738595"  #:
    steelblue = "#5a7d9a"  #:
    steelgrey = "#6f828a"  #:
    stone = "#ada587"  #:
    stormyblue = "#507b9c"  #:
    straw = "#fcf679"  #:
    strawberry = "#fb2943"  #:
    strongblue = "#0c06f7"  #:
    strongpink = "#ff0789"  #:
    sunflower = "#ffc512"  #:
    sunfloweryellow = "#ffda03"  #:
    sunnyyellow = "#fff917"  #:
    sunshineyellow = "#fffd37"  #:
    sunyellow = "#ffdf22"  #:
    swamp = "#698339"  #:
    swampgreen = "#748500"  #:
    tan = "#d1b26f"  #:
    tanbrown = "#ab7e4c"  #:
    tangerine = "#ff9408"  #:
    tangreen = "#a9be70"  #:
    taupe = "#b9a281"  #:
    tea = "#65ab7c"  #:
    teagreen = "#bdf8a3"  #:
    teal = "#029386"  #:
    tealblue = "#01889f"  #:
    tealgreen = "#25a36f"  #:
    tealish = "#24bca8"  #:
    tealishgreen = "#0cdc73"  #:
    terracota = "#cb6843"  #:
    terracotta = "#c9643b"  #:
    tiffanyblue = "#7bf2da"  #:
    tomato = "#ef4026"  #:
    tomatored = "#ec2d01"  #:
    topaz = "#13bbaf"  #:
    toupe = "#c7ac7d"  #:
    toxicgreen = "#61de2a"  #:
    treegreen = "#2a7e19"  #:
    trueblue = "#010fcc"  #:
    truegreen = "#089404"  #:
    turquoise = "#06c2ac"  #:
    turquoiseblue = "#06b1c4"  #:
    turquoisegreen = "#04f489"  #:
    turtlegreen = "#75b84f"  #:
    twilight = "#4e518b"  #:
    twilightblue = "#0a437a"  #:
    uglyblue = "#31668a"  #:
    uglybrown = "#7d7103"  #:
    uglygreen = "#7a9703"  #:
    uglypink = "#cd7584"  #:
    uglypurple = "#a442a0"  #:
    uglyyellow = "#d0c101"  #:
    ultramarine = "#2000b1"  #:
    ultramarineblue = "#1805db"  #:
    umber = "#b26400"  #:
    velvet = "#750851"  #:
    vermillion = "#f4320c"  #:
    verydarkblue = "#000133"  #:
    verydarkbrown = "#1d0200"  #:
    verydarkgreen = "#062e03"  #:
    verydarkpurple = "#2a0134"  #:
    verylightblue = "#d5ffff"  #:
    verylightbrown = "#d3b683"  #:
    verylightgreen = "#d1ffbd"  #:
    verylightpink = "#fff4f2"  #:
    verylightpurple = "#f6cefc"  #:
    verypaleblue = "#d6fffe"  #:
    verypalegreen = "#cffdbc"  #:
    vibrantblue = "#0339f8"  #:
    vibrantgreen = "#0add08"  #:
    vibrantpurple = "#ad03de"  #:
    violet = "#9a0eea"  #:
    violetblue = "#510ac9"  #:
    violetpink = "#fb5ffc"  #:
    violetred = "#a50055"  #:
    viridian = "#1e9167"  #:
    vividblue = "#152eff"  #:
    vividgreen = "#2fef10"  #:
    vividpurple = "#9900fa"  #:
    vomit = "#a2a415"  #:
    vomitgreen = "#89a203"  #:
    vomityellow = "#c7c10c"  #:
    warmblue = "#4b57db"  #:
    warmbrown = "#964e02"  #:
    warmgrey = "#978a84"  #:
    warmpink = "#fb5581"  #:
    warmpurple = "#952e8f"  #:
    washedoutgreen = "#bcf5a6"  #:
    waterblue = "#0e87cc"  #:
    watermelon = "#fd4659"  #:
    weirdgreen = "#3ae57f"  #:
    wheat = "#fbdd7e"  #:
    white = "#ffffff"  #:
    windowsblue = "#3778bf"  #:
    wine = "#80013f"  #:
    winered = "#7b0323"  #:
    wintergreen = "#20f986"  #:
    wisteria = "#a87dc2"  #:
    yellow = "#ffff14"  #:
    yellowbrown = "#b79400"  #:
    yellowgreen = "#bbf90f"  #:
    yellowish = "#faee66"  #:
    yellowishbrown = "#9b7a01"  #:
    yellowishgreen = "#b0dd16"  #:
    yellowishorange = "#ffab0f"  #:
    yellowishtan = "#fcfc81"  #:
    yellowochre = "#cb9d06"  #:
    yelloworange = "#fcb001"  #:
    yellowtan = "#ffe36e"  #:
    yellowybrown = "#ae8b0c"  #:
    yellowygreen = "#bff128"  #:



"""
    test_domonic
    ~~~~~~~~~~~~
    domonic core tests for __init__.py at the root of the domonic package

    (previously no tests. was using examples/parsing/page.py to drive dev)

"""

import unittest
# import requests
# from mock import patch

# from domonic.dom import *
from domonic import domonic


class TestCase(unittest.TestCase):
    """ Tests for the domonic """

    def test_load(self):
        t1 = domonic.load('<html></html>')
        print(t1)

    def test_loads(self):
        # t1 = domonic.loads('<html></html>')
        # print(t1)
        pass

    def parse(self):
        t1 = domonic.parse('<html></html>')
        assert t1 == "html(),"  # hmm wondering if parse is correct term. as returns pyml strings

        # t2 = domonic.parse('<html><body></body></html>')
        # print(t2)
        # t3 = domonic.parse('<html><body><p></p></body></html>')
        # print(t3)

        pass

    def evaluate(self):
        t1 = domonic.evaluate('<html></html>')
        print(t1)




if __name__ == '__main__':
    unittest.main()



"""
    test_terminal
    ~~~~~~~~~~~~~~~~
"""

import unittest

from domonic.terminal import *
from domonic.decorators import silence


class TestCase(unittest.TestCase):

    def test_bash_ls(self):
        files = ls()
        print(files)
        # return
        self.assertTrue('domonic' in ls())
        print(ls("-al"))
        print(ls("../"))
        for line in ls():
            print("line:", line)
        # for f in ls():
        #     try:
        #         print(f)
        #         print(cat(f))
        #     except Exception as e:
        #         pass

    def test_bash_pwd(self):
        thedir = pwd()
        print("OYI::", thedir)
        self.assertTrue('domonic' in thedir)

    def test_bash_cd(self):
        pass # TODO - need to change github action
        #print(cd('../'))  # < CD does not run on terminal
        #thedir_aftercd = pwd()
        #print(thedir_aftercd)
        #self.assertTrue('domonic' not in thedir_aftercd)
        #print(cd('domonic'))
        #thedir_aftercd = pwd()
        #print(thedir_aftercd)
        #self.assertTrue('domonic' in thedir_aftercd)

    def test_bash_mkdir(self):
        try:
            mkdir('somedir')
            self.assertTrue('somedir' in ls())
        except Exception as e:
            print(e)
        finally:
            # rm('-r somedir')
            rmdir('somedir')
            self.assertTrue('somedir' not in ls())

    def test_bash_touch(self):
        try:
            touch('somefile')
            self.assertTrue('somefile' in ls())
        except Exception as e:
            print(e)
        finally:
            rm('somefile')
            self.assertTrue('somefile' not in ls())

    @silence
    def test_bash_git(self):
        print(git('status'))
        self.assertTrue('master' in git('status'))

    def test_bash_general(self):
        print(man("ls"))
        print(echo('test'))
        print(df())
        print(du())
        print(ps())
        # print(cowsay('moo'))
        print(date())
        print(cal())
         # failing on github actions
        # for i, l in enumerate(cat('LICENSE.txt')):
            # print(i, l)

    def test_bash_history(self):
        pass # failing on github actions
        # print(history())
        # for i, thing in enumerate(history(), 1):
            # print(i, thing)

    @silence
    def test_bash(self):
        print("ran")
        print(ping('http://eventual.technology'))  # < TODO - need to strean output
        # print(wget('eventual.technology'))


if __name__ == '__main__':
    unittest.main()



"""
    test_domonic
    ~~~~~~~~~~~~
    unit tests for css
"""

import unittest
# import requests
# from mock import patch

from domonic.html import *
from domonic.style import *
from domonic.dom import *


class TestCase(unittest.TestCase):

    def test_domonic_css(self):

        atag = a("linky", _href="https://eventual.technology")
        print(atag.style.alignContent)

        sometag = div("asdfasdf", _id="test")
        print(sometag.style)
        sometag.style.alignContent = None
        print(sometag.style.alignContent)

        sometag.style.backgroundColor = "black"
        sometag.style.fontSize = "12px"

        # huh = document.createAttribute("test")
        # huh.value = "wtf"
        # sometag.setAttributeNode(huh)
        print(sometag.style.fontSize)
        print(sometag)

        # dom.select('#test' ).dostuff() # TODO -
        # print(sometag.style)
        # print(sometag.tagName)
        # s = Style()
        # print(sometag)


if __name__ == '__main__':
    unittest.main()



"""
    test_svg
    ~~~~~~~~~~~~
    - unit tests for svg
"""

import unittest
# import requests
# from mock import patch

from domonic import domonic

from domonic.svg import *
from domonic.decorators import silence


class TestCase(unittest.TestCase):

    @silence
    def test_domonic_svg(self):
        mysvg = svg()
        print(mysvg)

    def test_domonic_cirlce(self):
        test = svg(
            circle(_cx="50", _cy="50", _r="40", _stroke="green", **{"_stroke-width":"4"}, _fill="yellow"),
            _width="100", _height="100",
        )
        print(test)

    def test_domonic_node(self):
        circ = svg(
            circle(_cx="50", _cy="50", _r="40", _stroke="green", **{"_stroke-width": "4"}, _fill="yellow"),
            _width="100", _height="100",
        )
        mysvg = svg()
        mysvg.appendChild(circ / 10)
        print(mysvg)


if __name__ == '__main__':
    unittest.main()



"""
    domonic.d3
    ====================================

"""

from domonic.html import *


# from.domonic.d3.format import *
# from.domonic.d3.queue import *
# from.domonic.d3.array import *
# from.domonic.d3.axis import *
# from.domonic.d3.brush import *
# from.domonic.d3.chord import *
# from.domonic.d3.color import *
# from.domonic.d3.contour import *
# from.domonic.d3.delaunay import *
# from.domonic.d3.dispatch import *
# from.domonic.d3.drag import *
# from.domonic.d3.dsv import *
# from.domonic.d3.ease import *
# from.domonic.d3.fetch import *
# from.domonic.d3.force import *
# from.domonic.d3.format import *
# from.domonic.d3.geo import *
# from.domonic.d3.heirarchy import *
# from.domonic.d3.interpolate import *
# from.domonic.d3.path import *
# from.domonic.d3.polygon import *
# from.domonic.d3.quadtree import *
# from.domonic.d3.queue import *
# from.domonic.d3.random import *
# from.domonic.d3.scale-chromatic import *
# from.domonic.d3.scale import *
from domonic.d3.selection import *
# from.domonic.d3.shape import *
# from.domonic.d3.tile import *
# from.domonic.d3.time-format import *
# from.domonic.d3.time import *
# from.domonic.d3.timer import *
# from.domonic.d3.transition import *
# from.domonic.d3.zoom import *


class d3_el():
    """
    d3_el - contains the d3_el class and its methods
    """

    DOM = None

    def __init__(self, dom, *args, **kwargs):
        if type(dom) == str:
            raise ValueError('d3_el.__init__: dom must be a DOM object')
        else:
            d3_el.DOM = dom
            self.dom = dom

    def __str__(self):
        return str(self.dom)

    @property
    def dom(self):
        if d3_el.DOM is None:
            raise ValueError('d3_el.dom: DOM has not been set')
        return d3_el.DOM

    @dom.setter
    def dom(self, dom):
        if isinstance(dom, html) or isinstance(dom, Document):
            d3_el.DOM = dom

    def init(self, q=''):
        self.q = q
        if type(q) is not str:
            return
        # if q == "":
            # return

        if self.q[0] == '<':
            self.elements = domonic.domonic.load(self.q)
            if isinstance(self.elements, html) or isinstance(self.elements, Document):
                self.dom = self.elements
        else:
            try:
                # element by selector not working on just classes as always needs a tag
                if self.q[0] == '.':
                    self.elements = self.dom.querySelectorAll(self.q)
                    return
                self.elements = self.dom.getElementsBySelector(self.q, self.dom)
            except Exception as e:
                print('Error. No DOM has been set!!', e)
                raise e


def dproxy(q):
    el = d3_el(q)
    el.init(q)

    # if type(q) is not str:
    return el
    # else:
    #     return el.elements
    # def __str__(self):
    #     return self.elements


class d3(d3_el):

    def __init__(self, selector=None, *args, **kwargs):
        super().__init__(selector, *args, **kwargs)
        self.init(selector)

    def __call__(self, *args, **kwargs):
        return dproxy(args[0])



"""
    domonic.mathml
    ====================================
    Generate MATHML using python 3

"""

from domonic.html import tag
from domonic.dom import Node


mathml_tags = ["math", "maction", "math", "menclose", "merror", "mfenced", "mfrac", "mi", "mmultiscripts", "mn", "mo", "mover", "mpadded", "mphantom", "mroot", "mrow",
                "ms", "mspace", "msqrt", "mstyle", "msub", "msubsup", "msup", "mtable", "mtd", "mtext", "mtr", "munder", "munderover", "semantics", "maligngroup",
                "malignmark", "msline", "msgroup", "mlongdiv", "mstyle", "mprescripts", "mscarries", "mscarry", "munder", "munderover", "none"]

# mathml_attributes = []


def mathml_init(self, *args, **kwargs):
    tag.__init__(self, *args, **kwargs)
    Node.__init__(self, *args, **kwargs)


math_ = type('math', (tag, Node), {'name': 'math', '__init__': mathml_init})
maction = type('maction', (tag, Node), {'name': 'maction', '__init__': mathml_init})
menclose = type('menclose', (tag, Node), {'name': 'menclose', '__init__': mathml_init})
merror = type('merror', (tag, Node), {'name': 'merror', '__init__': mathml_init})
mfenced = type('mfenced', (tag, Node), {'name': 'mfenced', '__init__': mathml_init})
mfrac = type('mfrac', (tag, Node), {'name': 'mfrac', '__init__': mathml_init})
mi = type('mi', (tag, Node), {'name': 'mi', '__init__': mathml_init})
mmultiscripts = type('mmultiscripts', (tag, Node), {'name': 'mmultiscripts', '__init__': mathml_init})
mn = type('mn', (tag, Node), {'name': 'mn', '__init__': mathml_init})
mo = type('mo', (tag, Node), {'name': 'mo', '__init__': mathml_init})
mover = type('mover', (tag, Node), {'name': 'mover', '__init__': mathml_init})
mpadded = type('mpadded', (tag, Node), {'name': 'mpadded', '__init__': mathml_init})
mphantom = type('mphantom', (tag, Node), {'name': 'mphantom', '__init__': mathml_init})
mroot = type('mroot', (tag, Node), {'name': 'mroot', '__init__': mathml_init})
mrow = type('mrow', (tag, Node), {'name': 'mrow', '__init__': mathml_init})
ms = type('ms', (tag, Node), {'name': 'ms', '__init__': mathml_init})
mspace = type('mspace', (tag, Node), {'name': 'mspace', '__init__': mathml_init})
msqrt = type('msqrt', (tag, Node), {'name': 'msqrt', '__init__': mathml_init})
mstyle = type('mstyle', (tag, Node), {'name': 'mstyle', '__init__': mathml_init})
msub = type('msub', (tag, Node), {'name': 'msub', '__init__': mathml_init})
msubsup = type('msubsup', (tag, Node), {'name': 'msubsup', '__init__': mathml_init})
msup = type('msup', (tag, Node), {'name': 'msup', '__init__': mathml_init})
mtable = type('mtable', (tag, Node), {'name': 'mtable', '__init__': mathml_init})
mtd = type('mtd', (tag, Node), {'name': 'mtd', '__init__': mathml_init})
mtext = type('mtext', (tag, Node), {'name': 'mtext', '__init__': mathml_init})
mtr = type('mtr', (tag, Node), {'name': 'mtr', '__init__': mathml_init})
munder = type('munder', (tag, Node), {'name': 'munder', '__init__': mathml_init})
munderover = type('munderover', (tag, Node), {'name': 'munderover', '__init__': mathml_init})
semantics = type('semantics', (tag, Node), {'name': 'semantics', '__init__': mathml_init})
maligngroup = type('maligngroup', (tag, Node), {'name': 'maligngroup', '__init__': mathml_init})
malignmark = type('malignmark', (tag, Node), {'name': 'malignmark', '__init__': mathml_init})
msline = type('msline', (tag, Node), {'name': 'msline', '__init__': mathml_init})
msgroup = type('msgroup', (tag, Node), {'name': 'msgroup', '__init__': mathml_init})
mlongdiv = type('mlongdiv', (tag, Node), {'name': 'mlongdiv', '__init__': mathml_init})
mstyle = type('mstyle', (tag, Node), {'name': 'mstyle', '__init__': mathml_init})
mprescripts = type('mprescripts', (tag, Node), {'name': 'mprescripts', '__init__': mathml_init})
mscarries = type('mscarries', (tag, Node), {'name': 'mscarries', '__init__': mathml_init})
mscarry = type('mscarry', (tag, Node), {'name': 'mscarry', '__init__': mathml_init})
munder = type('munder', (tag, Node), {'name': 'munder', '__init__': mathml_init})
munderover = type('munderover', (tag, Node), {'name': 'munderover', '__init__': mathml_init})
none = type('none', (tag, Node), {'name': 'none', '__init__': mathml_init})



"""
    test_collada
    ~~~~~~~~~~~~
"""

import unittest
# import requests
# from mock import patch

from domonic import domonic

from domonic.dom import *
from domonic.html import *
from domonic.x3d import *
from domonic.decorators import silence


class TestCase(unittest.TestCase):

    # @silence
    def test_domonic_x3dom(self):
    	x3dom_test = html(
			head(
			    meta(**{"_http-equiv":"X-UA-Compatible"}, _content="IE=edge"),
			    title("My first X3DOM page"),
			    script(_type='text/javascript', _src='https://www.x3dom.org/download/x3dom.js'),
			    script(
		    	"""
				$(document).ready(function(){
				    var screenshotCount = 0;

				    //Every time the user clicks on the 'take screenhot' button
				    $("#btnTakeScreenshot").on("click", function() {
				        //Get data url from the runtime
				        var imgUrl = document.getElementById("canvas").runtime.getScreenshot();

				        //Create preview image...
				        var newScreenshotImage = document.createElement('img');
				        newScreenshotImage.src = imgUrl;
				        newScreenshotImage.id = "screenshot_" + screenshotCount;
				        $('#screenshotPreviews').append(newScreenshotImage);

				        //...and download link
				        var newScreenshotDownloadLink = document.createElement('a');
				        newScreenshotDownloadLink.href = imgUrl;
				        newScreenshotDownloadLink.download = "screenshot_" + screenshotCount + ".png";
				        newScreenshotDownloadLink.innerHTML = "Speichern";
				        $('#screenshotPreviews').append(newScreenshotDownloadLink);

				        screenshotCount++;
				        $('#screenshotCount').html(screenshotCount);
				    });
				});
		    	"""
			    ),
			    link(_rel='stylesheet', _type='text/css', _href='https://www.x3dom.org/download/x3dom.css')
			),
			body(
				h1("Animate Objects with X3DOM!"),
				p("Learn how to animate objects."),
				x3d(_width='500px', _height='400px').append(
			    	scene(
				        transform(_DEF="ball").append(
					        shape(
					            appearance(
					                material(_diffuseColor='1 0 0')
					            ),
					            sphere()
					        )
				        ),
			        	timeSensor(_DEF="time", _cycleInterval="2", _loop="true"),
			        	PositionInterpolator(_DEF="move", _key="0 0.5 1", _keyValue="0 0 0  0 3 0  0 0 0"),
			        	Route(_fromNode="time", _fromField ="fraction_changed", _toNode="move", _toField="set_fraction"),
			        	Route(_fromNode="move", _fromField ="value_changed", _toNode="ball", _toField="translation")
			        )
			    )
			)
		)

    	render( str(x3dom_test) )#, "sphere_test.html" )


if __name__ == '__main__':
    unittest.main()



"""
    domonic.collada
    ====================================
    Generate Collada with python 3

"""

from domonic.html import tag, closed_tag
from domonic.dom import Node


def write(inp, outp='output'):
    """write

    Renders the input to string or .dae file

    Args:
        inp (obj): domonic.collada tags
        outp (str): An optional filename for the .dae file that gets produced

    Returns:
        str: A HTML rendered string
    """
    if outp != '':
        with open(outp, "w+") as f:
            f.write(str(inp) + ".dae")
    return str(inp)


def collada_init(self, *args, **kwargs):
    tag.__init__(self, *args, **kwargs)
    Node.__init__(self, *args, **kwargs)


COLLADA = type('COLLADA', (tag, Node), {'name': 'COLLADA', '__init__': collada_init})

asset = type('asset', (tag, Node), {'name': 'asset', '__init__': collada_init})
library_cameras = type('library_cameras', (tag, Node), {'name': 'library_cameras', '__init__': collada_init})
library_lights = type('library_lights', (tag, Node), {'name': 'library_lights', '__init__': collada_init})
library_materials = type('library_materials', (tag, Node), {'name': 'library_materials', '__init__': collada_init})
library_effects = type('library_effects', (tag, Node), {'name': 'library_effects', '__init__': collada_init})
library_geometries = type('library_geometries', (tag, Node), {'name': 'library_geometries', '__init__': collada_init})
library_visual_scenes = type('library_visual_scenes', (tag, Node), {'name': 'library_visual_scenes', '__init__': collada_init})
scene = type('scene', (tag, Node), {'name': 'scene', '__init__': collada_init})
extra = type('extra', (tag, Node), {'name': 'extra', '__init__': collada_init})
author = type('author', (tag, Node), {'name': 'author', '__init__': collada_init})
up_axis = type('up_axis', (tag, Node), {'name': 'up_axis', '__init__': collada_init})
camera = type('camera', (tag, Node), {'name': 'camera', '__init__': collada_init})
optics = type('optics', (tag, Node), {'name': 'optics', '__init__': collada_init})
technique = type('technique', (tag, Node), {'name': 'technique', '__init__': collada_init})
technique_common = type('technique_common', (tag, Node), {'name': 'technique_common', '__init__': collada_init})
aspect_ratio = type('aspect_ratio', (tag, Node), {'name': 'aspect_ratio', '__init__': collada_init})
light = type('light', (tag, Node), {'name': 'light', '__init__': collada_init})
constant_attenuation = type('constant_attenuation', (tag, Node), {'name': 'constant_attenuation', '__init__': collada_init})
linear_attenuation = type('linear_attenuation', (tag, Node), {'name': 'linear_attenuation', '__init__': collada_init})
quadratic_attenuation = type('quadratic_attenuation', (tag, Node), {'name': 'quadratic_attenuation', '__init__': collada_init})
material = type('material', (tag, Node), {'name': 'material', '__init__': collada_init})
effect = type('effect', (tag, Node), {'name': 'effect', '__init__': collada_init})
profile_COMMON = type('profile_COMMON', (tag, Node), {'name': 'profile_COMMON', '__init__': collada_init})
phong = type('phong', (tag, Node), {'name': 'phong', '__init__': collada_init})
emission = type('emission', (tag, Node), {'name': 'emission', '__init__': collada_init})
geometry = type('geometry', (tag, Node), {'name': 'geometry', '__init__': collada_init})
mesh = type('mesh', (tag, Node), {'name': 'mesh', '__init__': collada_init})
source = type('source', (tag, Node), {'name': 'source', '__init__': collada_init})
vertices = type('vertices', (tag, Node), {'name': 'vertices', '__init__': collada_init})
polylist = type('polylist', (tag, Node), {'name': 'polylist', '__init__': collada_init})
vcount = type('vcount', (tag, Node), {'name': 'vcount', '__init__': collada_init})
p = type('p', (tag, Node), {'name': 'p', '__init__': collada_init})
library_visual_scenes = type('library_visual_scenes', (tag, Node), {'name': 'library_visual_scenes', '__init__': collada_init})
visual_scene = type('visual_scene', (tag, Node), {'name': 'visual_scene', '__init__': collada_init})
node = type('node', (tag, Node), {'name': 'node', '__init__': collada_init})
instance_visual_scene = type('instance_visual_scene', (tag, Node), {'name': 'instance_visual_scene', '__init__': collada_init})
test_element = type('test_element', (tag, Node), {'name': 'test_element', '__init__': collada_init})
instance_effect = type('instance_effect', (closed_tag, Node), {'name': 'instance_effect', '__init__': collada_init})
texture_unit = type('texture_unit', (closed_tag, Node), {'name': 'texture_unit', '__init__': collada_init})
tapered_cylinder = type('tapered_cylinder', (closed_tag, Node), {'name': 'tapered_cylinder', '__init__': collada_init})
tapered_capsule = type('tapered_capsule', (closed_tag, Node), {'name': 'tapered_capsule', '__init__': collada_init})
format_hint = type('format_hint', (closed_tag, Node), {'name': 'format_hint', '__init__': collada_init})
revisions = type('revisions', (closed_tag, Node), {'name': 'revisions', '__init__': collada_init})
contributer = type('contributer', (closed_tag, Node), {'name': 'contributer', '__init__': collada_init})
acceleration = type('acceleration', (tag, Node), {'name': 'acceleration', '__init__': collada_init})
axis_info = type('axis_info', (tag, Node), {'name': 'axis_info', '__init__': collada_init})
accessor = type('accessor', (tag, Node), {'name': 'accessor', '__init__': collada_init})
active = type('active', (tag, Node), {'name': 'active', '__init__': collada_init})
alpha_func = type('alpha_func', (tag, Node), {'name': 'alpha_func', '__init__': collada_init})
alpha_test_enable = type('alpha_test_enable', (tag, Node), {'name': 'alpha_test_enable', '__init__': collada_init})
alpha = type('alpha', (tag, Node), {'name': 'alpha', '__init__': collada_init})
altitude = type('altitude', (tag, Node), {'name': 'altitude', '__init__': collada_init})
ambient = type('ambient', (tag, Node), {'name': 'ambient', '__init__': collada_init})
angle = type('angle', (tag, Node), {'name': 'angle', '__init__': collada_init})
angular_velocity = type('angular_velocity', (tag, Node), {'name': 'angular_velocity', '__init__': collada_init})
angular = type('angular', (tag, Node), {'name': 'angular', '__init__': collada_init})
animation_clip = type('animation_clip', (tag, Node), {'name': 'animation_clip', '__init__': collada_init})
animation = type('animation', (tag, Node), {'name': 'animation', '__init__': collada_init})
annotate = type('annotate', (tag, Node), {'name': 'annotate', '__init__': collada_init})
argument = type('argument', (tag, Node), {'name': 'argument', '__init__': collada_init})
array = type('array', (tag, Node), {'name': 'array', '__init__': collada_init})
articulated_system = type('articulated_system', (tag, Node), {'name': 'articulated_system', '__init__': collada_init})
perspective = type('perspective', (tag, Node), {'name': 'perspective', '__init__': collada_init})
asset = type('asset', (tag, Node), {'name': 'asset', '__init__': collada_init})
se = type('se', (tag, Node), {'name': 'se', '__init__': collada_init})
attachment_end = type('attachment_end', (tag, Node), {'name': 'attachment_end', '__init__': collada_init})
attachment_full = type('attachment_full', (tag, Node), {'name': 'attachment_full', '__init__': collada_init})
attachment_start = type('attachment_start', (tag, Node), {'name': 'attachment_start', '__init__': collada_init})
attachment = type('attachment', (tag, Node), {'name': 'attachment', '__init__': collada_init})
author_email = type('author_email', (tag, Node), {'name': 'author_email', '__init__': collada_init})
author_website = type('author_website', (tag, Node), {'name': 'author_website', '__init__': collada_init})
author = type('author', (tag, Node), {'name': 'author', '__init__': collada_init})
authoring_tool = type('authoring_tool', (tag, Node), {'name': 'authoring_tool', '__init__': collada_init})
auto_normal_enable = type('auto_normal_enable', (tag, Node), {'name': 'auto_normal_enable', '__init__': collada_init})
axis_info = type('axis_info', (tag, Node), {'name': 'axis_info', '__init__': collada_init})
axis = type('axis', (tag, Node), {'name': 'axis', '__init__': collada_init})
prismatic = type('prismatic', (tag, Node), {'name': 'prismatic', '__init__': collada_init})
revolute = type('revolute', (tag, Node), {'name': 'revolute', '__init__': collada_init})
swept_surface = type('swept_surface', (tag, Node), {'name': 'swept_surface', '__init__': collada_init})
binary = type('binary', (tag, Node), {'name': 'binary', '__init__': collada_init})
bind_attribute = type('bind_attribute', (tag, Node), {'name': 'bind_attribute', '__init__': collada_init})
bind_joint_axis = type('bind_joint_axis', (tag, Node), {'name': 'bind_joint_axis', '__init__': collada_init})
bind_kinematics_model = type('bind_kinematics_model', (tag, Node), {'name': 'bind_kinematics_model', '__init__': collada_init})
bind_shape_matrix = type('bind_shape_matrix', (tag, Node), {'name': 'bind_shape_matrix', '__init__': collada_init})
bind_uniform = type('bind_uniform', (tag, Node), {'name': 'bind_uniform', '__init__': collada_init})
bind_vertex_input = type('bind_vertex_input', (tag, Node), {'name': 'bind_vertex_input', '__init__': collada_init})
bind = type('bind', (tag, Node), {'name': 'bind', '__init__': collada_init})
bind = type('bind', (tag, Node), {'name': 'bind', '__init__': collada_init})
blend_color = type('blend_color', (tag, Node), {'name': 'blend_color', '__init__': collada_init})
blend_enable = type('blend_enable', (tag, Node), {'name': 'blend_enable', '__init__': collada_init})
blend_equation_separate = type('blend_equation_separate', (tag, Node), {'name': 'blend_equation_separate', '__init__': collada_init})
blend_equation = type('blend_equation', (tag, Node), {'name': 'blend_equation', '__init__': collada_init})
blend_func_separate = type('blend_func_separate', (tag, Node), {'name': 'blend_func_separate', '__init__': collada_init})
blend_func = type('blend_func', (tag, Node), {'name': 'blend_func', '__init__': collada_init})
blinn = type('blinn', (tag, Node), {'name': 'blinn', '__init__': collada_init})
bool_array = type('bool_array', (tag, Node), {'name': 'bool_array', '__init__': collada_init})
border_color = type('border_color', (tag, Node), {'name': 'border_color', '__init__': collada_init})
box = type('box', (tag, Node), {'name': 'box', '__init__': collada_init})
brep = type('brep', (tag, Node), {'name': 'brep', '__init__': collada_init})
camera = type('camera', (tag, Node), {'name': 'camera', '__init__': collada_init})
capsule = type('capsule', (tag, Node), {'name': 'capsule', '__init__': collada_init})
channel = type('channel', (tag, Node), {'name': 'channel', '__init__': collada_init})
arget = type('arget', (tag, Node), {'name': 'arget', '__init__': collada_init})
circle = type('circle', (tag, Node), {'name': 'circle', '__init__': collada_init})
clip_plane_enable = type('clip_plane_enable', (tag, Node), {'name': 'clip_plane_enable', '__init__': collada_init})
clip_plane = type('clip_plane', (tag, Node), {'name': 'clip_plane', '__init__': collada_init})
code = type('code', (tag, Node), {'name': 'code', '__init__': collada_init})
color_clear = type('color_clear', (tag, Node), {'name': 'color_clear', '__init__': collada_init})
color_logic_op_enable = type('color_logic_op_enable', (tag, Node), {'name': 'color_logic_op_enable', '__init__': collada_init})
color_mask = type('color_mask', (tag, Node), {'name': 'color_mask', '__init__': collada_init})
color_material_enable = type('color_material_enable', (tag, Node), {'name': 'color_material_enable', '__init__': collada_init})
color_material = type('color_material', (tag, Node), {'name': 'color_material', '__init__': collada_init})
color_target = type('color_target', (tag, Node), {'name': 'color_target', '__init__': collada_init})
color = type('color', (tag, Node), {'name': 'color', '__init__': collada_init})
comments = type('comments', (tag, Node), {'name': 'comments', '__init__': collada_init})
compiler = type('compiler', (tag, Node), {'name': 'compiler', '__init__': collada_init})
cone = type('cone', (tag, Node), {'name': 'cone', '__init__': collada_init})
connect_param = type('connect_param', (tag, Node), {'name': 'connect_param', '__init__': collada_init})
point = type('point', (tag, Node), {'name': 'point', '__init__': collada_init})
spot = type('spot', (tag, Node), {'name': 'spot', '__init__': collada_init})
constant = type('constant', (tag, Node), {'name': 'constant', '__init__': collada_init})
constant = type('constant', (tag, Node), {'name': 'constant', '__init__': collada_init})
contributor = type('contributor', (tag, Node), {'name': 'contributor', '__init__': collada_init})
control_vertices = type('control_vertices', (tag, Node), {'name': 'control_vertices', '__init__': collada_init})
controller = type('controller', (tag, Node), {'name': 'controller', '__init__': collada_init})
convex_mesh = type('convex_mesh', (tag, Node), {'name': 'convex_mesh', '__init__': collada_init})
copyright = type('copyright', (tag, Node), {'name': 'copyright', '__init__': collada_init})
coverage = type('coverage', (tag, Node), {'name': 'coverage', '__init__': collada_init})
create_2d = type('create_2d', (tag, Node), {'name': 'create_2d', '__init__': collada_init})
create_3d = type('create_3d', (tag, Node), {'name': 'create_3d', '__init__': collada_init})
create_cube = type('create_cube', (tag, Node), {'name': 'create_cube', '__init__': collada_init})
created = type('created', (tag, Node), {'name': 'created', '__init__': collada_init})
cull_face_enable = type('cull_face_enable', (tag, Node), {'name': 'cull_face_enable', '__init__': collada_init})
cull_face = type('cull_face', (tag, Node), {'name': 'cull_face', '__init__': collada_init})
curve = type('curve', (tag, Node), {'name': 'curve', '__init__': collada_init})
curves = type('curves', (tag, Node), {'name': 'curves', '__init__': collada_init})
cylinder = type('cylinder', (tag, Node), {'name': 'cylinder', '__init__': collada_init})
cylinder = type('cylinder', (tag, Node), {'name': 'cylinder', '__init__': collada_init})
damping = type('damping', (tag, Node), {'name': 'damping', '__init__': collada_init})
deceleration = type('deceleration', (tag, Node), {'name': 'deceleration', '__init__': collada_init})
axis_info = type('axis_info', (tag, Node), {'name': 'axis_info', '__init__': collada_init})
effector_info = type('effector_info', (tag, Node), {'name': 'effector_info', '__init__': collada_init})
density = type('density', (tag, Node), {'name': 'density', '__init__': collada_init})
depth_bounds_enable = type('depth_bounds_enable', (tag, Node), {'name': 'depth_bounds_enable', '__init__': collada_init})
depth_bounds = type('depth_bounds', (tag, Node), {'name': 'depth_bounds', '__init__': collada_init})
depth_clamp_enable = type('depth_clamp_enable', (tag, Node), {'name': 'depth_clamp_enable', '__init__': collada_init})
depth_clear = type('depth_clear', (tag, Node), {'name': 'depth_clear', '__init__': collada_init})
depth_func = type('depth_func', (tag, Node), {'name': 'depth_func', '__init__': collada_init})
depth_mask = type('depth_mask', (tag, Node), {'name': 'depth_mask', '__init__': collada_init})
depth_range = type('depth_range', (tag, Node), {'name': 'depth_range', '__init__': collada_init})
depth_target = type('depth_target', (tag, Node), {'name': 'depth_target', '__init__': collada_init})
depth_test_enable = type('depth_test_enable', (tag, Node), {'name': 'depth_test_enable', '__init__': collada_init})
diffuse = type('diffuse', (tag, Node), {'name': 'diffuse', '__init__': collada_init})
direction = type('direction', (tag, Node), {'name': 'direction', '__init__': collada_init})
line = type('line', (tag, Node), {'name': 'line', '__init__': collada_init})
swept_surface = type('swept_surface', (tag, Node), {'name': 'swept_surface', '__init__': collada_init})
directional = type('directional', (tag, Node), {'name': 'directional', '__init__': collada_init})
dither_enable = type('dither_enable', (tag, Node), {'name': 'dither_enable', '__init__': collada_init})
draw = type('draw', (tag, Node), {'name': 'draw', '__init__': collada_init})
dynamic_friction = type('dynamic_friction', (tag, Node), {'name': 'dynamic_friction', '__init__': collada_init})
dynamic = type('dynamic', (tag, Node), {'name': 'dynamic', '__init__': collada_init})
edges = type('edges', (tag, Node), {'name': 'edges', '__init__': collada_init})
effect = type('effect', (tag, Node), {'name': 'effect', '__init__': collada_init})
effector_info = type('effector_info', (tag, Node), {'name': 'effector_info', '__init__': collada_init})
ellipse = type('ellipse', (tag, Node), {'name': 'ellipse', '__init__': collada_init})
emission = type('emission', (tag, Node), {'name': 'emission', '__init__': collada_init})
enabled = type('enabled', (tag, Node), {'name': 'enabled', '__init__': collada_init})
equation = type('equation', (tag, Node), {'name': 'equation', '__init__': collada_init})
evaluate_scene = type('evaluate_scene', (tag, Node), {'name': 'evaluate_scene', '__init__': collada_init})
evaluate = type('evaluate', (tag, Node), {'name': 'evaluate', '__init__': collada_init})
exact = type('exact', (tag, Node), {'name': 'exact', '__init__': collada_init})
extra = type('extra', (tag, Node), {'name': 'extra', '__init__': collada_init})
faces = type('faces', (tag, Node), {'name': 'faces', '__init__': collada_init})
falloff_angle = type('falloff_angle', (tag, Node), {'name': 'falloff_angle', '__init__': collada_init})
falloff_exponent = type('falloff_exponent', (tag, Node), {'name': 'falloff_exponent', '__init__': collada_init})
float_array = type('float_array', (tag, Node), {'name': 'float_array', '__init__': collada_init})
focal = type('focal', (tag, Node), {'name': 'focal', '__init__': collada_init})
fog_color = type('fog_color', (tag, Node), {'name': 'fog_color', '__init__': collada_init})
fog_coord_src = type('fog_coord_src', (tag, Node), {'name': 'fog_coord_src', '__init__': collada_init})
fog_density = type('fog_density', (tag, Node), {'name': 'fog_density', '__init__': collada_init})
fog_enable = type('fog_enable', (tag, Node), {'name': 'fog_enable', '__init__': collada_init})
fog_end = type('fog_end', (tag, Node), {'name': 'fog_end', '__init__': collada_init})
fog_mode = type('fog_mode', (tag, Node), {'name': 'fog_mode', '__init__': collada_init})
fog_state = type('fog_state', (tag, Node), {'name': 'fog_state', '__init__': collada_init})
force_field = type('force_field', (tag, Node), {'name': 'force_field', '__init__': collada_init})
formula = type('formula', (tag, Node), {'name': 'formula', '__init__': collada_init})
frame_object = type('frame_object', (tag, Node), {'name': 'frame_object', '__init__': collada_init})
frame_origin = type('frame_origin', (tag, Node), {'name': 'frame_origin', '__init__': collada_init})
frame_tcp = type('frame_tcp', (tag, Node), {'name': 'frame_tcp', '__init__': collada_init})
frame_tip = type('frame_tip', (tag, Node), {'name': 'frame_tip', '__init__': collada_init})
front_face = type('front_face', (tag, Node), {'name': 'front_face', '__init__': collada_init})
func = type('func', (tag, Node), {'name': 'func', '__init__': collada_init})
geographic_location = type('geographic_location', (tag, Node), {'name': 'geographic_location', '__init__': collada_init})
geometry = type('geometry', (tag, Node), {'name': 'geometry', '__init__': collada_init})
gravity = type('gravity', (tag, Node), {'name': 'gravity', '__init__': collada_init})
h = type('h', (tag, Node), {'name': 'h', '__init__': collada_init})
half_extents = type('half_extents', (tag, Node), {'name': 'half_extents', '__init__': collada_init})
height = type('height', (tag, Node), {'name': 'height', '__init__': collada_init})
capsule = type('capsule', (tag, Node), {'name': 'capsule', '__init__': collada_init})
binary = type('binary', (tag, Node), {'name': 'binary', '__init__': collada_init})
init_from = type('init_from', (tag, Node), {'name': 'init_from', '__init__': collada_init})
hint = type('hint', (tag, Node), {'name': 'hint', '__init__': collada_init})
hollow = type('hollow', (tag, Node), {'name': 'hollow', '__init__': collada_init})
IDREF_array = type('IDREF_array', (tag, Node), {'name': 'IDREF_array', '__init__': collada_init})
image = type('image', (tag, Node), {'name': 'image', '__init__': collada_init})
imager = type('imager', (tag, Node), {'name': 'imager', '__init__': collada_init})
include = type('include', (tag, Node), {'name': 'include', '__init__': collada_init})
index_of_refraction = type('index_of_refraction', (tag, Node), {'name': 'index_of_refraction', '__init__': collada_init})
index = type('index', (tag, Node), {'name': 'index', '__init__': collada_init})
inertia = type('inertia', (tag, Node), {'name': 'inertia', '__init__': collada_init})
instance_rigid_body = type('instance_rigid_body', (tag, Node), {'name': 'instance_rigid_body', '__init__': collada_init})
init_from = type('init_from', (tag, Node), {'name': 'init_from', '__init__': collada_init})
inline = type('inline', (tag, Node), {'name': 'inline', '__init__': collada_init})
emantic = type('emantic', (tag, Node), {'name': 'emantic', '__init__': collada_init})
emantics = type('emantics', (tag, Node), {'name': 'emantics', '__init__': collada_init})
instance_animation = type('instance_animation', (tag, Node), {'name': 'instance_animation', '__init__': collada_init})
instance_articulated_system = type('instance_articulated_system', (tag, Node), {'name': 'instance_articulated_system', '__init__': collada_init})
instance_camera = type('instance_camera', (tag, Node), {'name': 'instance_camera', '__init__': collada_init})
instance_controller = type('instance_controller', (tag, Node), {'name': 'instance_controller', '__init__': collada_init})
instance_effect = type('instance_effect', (tag, Node), {'name': 'instance_effect', '__init__': collada_init})
instance_force_field = type('instance_force_field', (tag, Node), {'name': 'instance_force_field', '__init__': collada_init})
instance_formula = type('instance_formula', (tag, Node), {'name': 'instance_formula', '__init__': collada_init})
instance_geometry = type('instance_geometry', (tag, Node), {'name': 'instance_geometry', '__init__': collada_init})
instance_image = type('instance_image', (tag, Node), {'name': 'instance_image', '__init__': collada_init})
instance_joint = type('instance_joint', (tag, Node), {'name': 'instance_joint', '__init__': collada_init})
instance_kinematics_model = type('instance_kinematics_model', (tag, Node), {'name': 'instance_kinematics_model', '__init__': collada_init})
instance_kinematics_scene = type('instance_kinematics_scene', (tag, Node), {'name': 'instance_kinematics_scene', '__init__': collada_init})
instance_light = type('instance_light', (tag, Node), {'name': 'instance_light', '__init__': collada_init})
instance_material = type('instance_material', (tag, Node), {'name': 'instance_material', '__init__': collada_init})
instance_material = type('instance_material', (tag, Node), {'name': 'instance_material', '__init__': collada_init})
instance_node = type('instance_node', (tag, Node), {'name': 'instance_node', '__init__': collada_init})
instance_physics_material = type('instance_physics_material', (tag, Node), {'name': 'instance_physics_material', '__init__': collada_init})
instance_physics_model = type('instance_physics_model', (tag, Node), {'name': 'instance_physics_model', '__init__': collada_init})
instance_physics_scene = type('instance_physics_scene', (tag, Node), {'name': 'instance_physics_scene', '__init__': collada_init})
instance_rigid_body = type('instance_rigid_body', (tag, Node), {'name': 'instance_rigid_body', '__init__': collada_init})
instance_rigid_constraint = type('instance_rigid_constraint', (tag, Node), {'name': 'instance_rigid_constraint', '__init__': collada_init})
instance_visual_scene = type('instance_visual_scene', (tag, Node), {'name': 'instance_visual_scene', '__init__': collada_init})
int_array = type('int_array', (tag, Node), {'name': 'int_array', '__init__': collada_init})
interpenetrate = type('interpenetrate', (tag, Node), {'name': 'interpenetrate', '__init__': collada_init})
jerk = type('jerk', (tag, Node), {'name': 'jerk', '__init__': collada_init})
axis_info = type('axis_info', (tag, Node), {'name': 'axis_info', '__init__': collada_init})
joint = type('joint', (tag, Node), {'name': 'joint', '__init__': collada_init})
joints = type('joints', (tag, Node), {'name': 'joints', '__init__': collada_init})
keywords = type('keywords', (tag, Node), {'name': 'keywords', '__init__': collada_init})
kinematics_model = type('kinematics_model', (tag, Node), {'name': 'kinematics_model', '__init__': collada_init})
kinematics_scene = type('kinematics_scene', (tag, Node), {'name': 'kinematics_scene', '__init__': collada_init})
kinematics = type('kinematics', (tag, Node), {'name': 'kinematics', '__init__': collada_init})
lambert = type('lambert', (tag, Node), {'name': 'lambert', '__init__': collada_init})
latitude = type('latitude', (tag, Node), {'name': 'latitude', '__init__': collada_init})
layer = type('layer', (tag, Node), {'name': 'layer', '__init__': collada_init})
library_animation_clips = type('library_animation_clips', (tag, Node), {'name': 'library_animation_clips', '__init__': collada_init})
library_animations = type('library_animations', (tag, Node), {'name': 'library_animations', '__init__': collada_init})
library_articulated_systems = type('library_articulated_systems', (tag, Node), {'name': 'library_articulated_systems', '__init__': collada_init})
library_cameras = type('library_cameras', (tag, Node), {'name': 'library_cameras', '__init__': collada_init})
library_controllers = type('library_controllers', (tag, Node), {'name': 'library_controllers', '__init__': collada_init})
library_effects = type('library_effects', (tag, Node), {'name': 'library_effects', '__init__': collada_init})
library_force_fields = type('library_force_fields', (tag, Node), {'name': 'library_force_fields', '__init__': collada_init})
library_formulas = type('library_formulas', (tag, Node), {'name': 'library_formulas', '__init__': collada_init})
library_geometries = type('library_geometries', (tag, Node), {'name': 'library_geometries', '__init__': collada_init})
library_images = type('library_images', (tag, Node), {'name': 'library_images', '__init__': collada_init})
library_joints = type('library_joints', (tag, Node), {'name': 'library_joints', '__init__': collada_init})
library_kinematics_models = type('library_kinematics_models', (tag, Node), {'name': 'library_kinematics_models', '__init__': collada_init})
library_kinematics_scenes = type('library_kinematics_scenes', (tag, Node), {'name': 'library_kinematics_scenes', '__init__': collada_init})
library_nodes = type('library_nodes', (tag, Node), {'name': 'library_nodes', '__init__': collada_init})
library_physics_materials = type('library_physics_materials', (tag, Node), {'name': 'library_physics_materials', '__init__': collada_init})
library_physics_models = type('library_physics_models', (tag, Node), {'name': 'library_physics_models', '__init__': collada_init})
library_physics_scenes = type('library_physics_scenes', (tag, Node), {'name': 'library_physics_scenes', '__init__': collada_init})
library_visual_scenes = type('library_visual_scenes', (tag, Node), {'name': 'library_visual_scenes', '__init__': collada_init})
light_ambient = type('light_ambient', (tag, Node), {'name': 'light_ambient', '__init__': collada_init})
light_constant_attenuation = type('light_constant_attenuation', (tag, Node), {'name': 'light_constant_attenuation', '__init__': collada_init})
light_diffuse = type('light_diffuse', (tag, Node), {'name': 'light_diffuse', '__init__': collada_init})
light_enable = type('light_enable', (tag, Node), {'name': 'light_enable', '__init__': collada_init})
light_linear_attenuation = type('light_linear_attenuation', (tag, Node), {'name': 'light_linear_attenuation', '__init__': collada_init})
light_model_ambient = type('light_model_ambient', (tag, Node), {'name': 'light_model_ambient', '__init__': collada_init})
light_model_color_control = type('light_model_color_control', (tag, Node), {'name': 'light_model_color_control', '__init__': collada_init})
light_model_local_viewer_enable = type('light_model_local_viewer_enable', (tag, Node), {'name': 'light_model_local_viewer_enable', '__init__': collada_init})
light_model_two_side_enable = type('light_model_two_side_enable', (tag, Node), {'name': 'light_model_two_side_enable', '__init__': collada_init})
light_position = type('light_position', (tag, Node), {'name': 'light_position', '__init__': collada_init})
light_quadratic_attenuation = type('light_quadratic_attenuation', (tag, Node), {'name': 'light_quadratic_attenuation', '__init__': collada_init})
light_specular = type('light_specular', (tag, Node), {'name': 'light_specular', '__init__': collada_init})
light_spot_cutoff = type('light_spot_cutoff', (tag, Node), {'name': 'light_spot_cutoff', '__init__': collada_init})
light_spot_direction = type('light_spot_direction', (tag, Node), {'name': 'light_spot_direction', '__init__': collada_init})
light_spot_exponent = type('light_spot_exponent', (tag, Node), {'name': 'light_spot_exponent', '__init__': collada_init})
lighting_enable = type('lighting_enable', (tag, Node), {'name': 'lighting_enable', '__init__': collada_init})
lights = type('lights', (tag, Node), {'name': 'lights', '__init__': collada_init})
limits = type('limits', (tag, Node), {'name': 'limits', '__init__': collada_init})
axis_info = type('axis_info', (tag, Node), {'name': 'axis_info', '__init__': collada_init})
prismatic = type('prismatic', (tag, Node), {'name': 'prismatic', '__init__': collada_init})
revolute = type('revolute', (tag, Node), {'name': 'revolute', '__init__': collada_init})
line_smooth_enable = type('line_smooth_enable', (tag, Node), {'name': 'line_smooth_enable', '__init__': collada_init})
line_stipple_enable = type('line_stipple_enable', (tag, Node), {'name': 'line_stipple_enable', '__init__': collada_init})
line_stipple = type('line_stipple', (tag, Node), {'name': 'line_stipple', '__init__': collada_init})
line_width = type('line_width', (tag, Node), {'name': 'line_width', '__init__': collada_init})
line = type('line', (tag, Node), {'name': 'line', '__init__': collada_init})
linear_attenuation = type('linear_attenuation', (tag, Node), {'name': 'linear_attenuation', '__init__': collada_init})
spot = type('spot', (tag, Node), {'name': 'spot', '__init__': collada_init})
linear = type('linear', (tag, Node), {'name': 'linear', '__init__': collada_init})
lines = type('lines', (tag, Node), {'name': 'lines', '__init__': collada_init})
linestrips = type('linestrips', (tag, Node), {'name': 'linestrips', '__init__': collada_init})
link = type('link', (tag, Node), {'name': 'link', '__init__': collada_init})
linker = type('linker', (tag, Node), {'name': 'linker', '__init__': collada_init})
locked = type('locked', (tag, Node), {'name': 'locked', '__init__': collada_init})
logic_op_enable = type('logic_op_enable', (tag, Node), {'name': 'logic_op_enable', '__init__': collada_init})
logic_op = type('logic_op', (tag, Node), {'name': 'logic_op', '__init__': collada_init})
longitude = type('longitude', (tag, Node), {'name': 'longitude', '__init__': collada_init})
lookout = type('lookout', (tag, Node), {'name': 'lookout', '__init__': collada_init})
magfilter = type('magfilter', (tag, Node), {'name': 'magfilter', '__init__': collada_init})
mass_frame = type('mass_frame', (tag, Node), {'name': 'mass_frame', '__init__': collada_init})
rigid_body = type('rigid_body', (tag, Node), {'name': 'rigid_body', '__init__': collada_init})
mass = type('mass', (tag, Node), {'name': 'mass', '__init__': collada_init})
shape = type('shape', (tag, Node), {'name': 'shape', '__init__': collada_init})
material_ambient = type('material_ambient', (tag, Node), {'name': 'material_ambient', '__init__': collada_init})
material_diffuse = type('material_diffuse', (tag, Node), {'name': 'material_diffuse', '__init__': collada_init})
material_emission = type('material_emission', (tag, Node), {'name': 'material_emission', '__init__': collada_init})
material_shininess = type('material_shininess', (tag, Node), {'name': 'material_shininess', '__init__': collada_init})
material_specular = type('material_specular', (tag, Node), {'name': 'material_specular', '__init__': collada_init})
material = type('material', (tag, Node), {'name': 'material', '__init__': collada_init})
matrix = type('matrix', (tag, Node), {'name': 'matrix', '__init__': collada_init})
max_anisotropy = type('max_anisotropy', (tag, Node), {'name': 'max_anisotropy', '__init__': collada_init})
mesh = type('mesh', (tag, Node), {'name': 'mesh', '__init__': collada_init})
minfilter = type('minfilter', (tag, Node), {'name': 'minfilter', '__init__': collada_init})
mip_bias = type('mip_bias', (tag, Node), {'name': 'mip_bias', '__init__': collada_init})
mip_max_level = type('mip_max_level', (tag, Node), {'name': 'mip_max_level', '__init__': collada_init})
mip_min_level = type('mip_min_level', (tag, Node), {'name': 'mip_min_level', '__init__': collada_init})
mipfilter = type('mipfilter', (tag, Node), {'name': 'mipfilter', '__init__': collada_init})
mips = type('mips', (tag, Node), {'name': 'mips', '__init__': collada_init})
create_cube = type('create_cube', (tag, Node), {'name': 'create_cube', '__init__': collada_init})
create2d = type('create2d', (tag, Node), {'name': 'create2d', '__init__': collada_init})
create3d = type('create3d', (tag, Node), {'name': 'create3d', '__init__': collada_init})
model_view_matrix = type('model_view_matrix', (tag, Node), {'name': 'model_view_matrix', '__init__': collada_init})
modified = type('modified', (tag, Node), {'name': 'modified', '__init__': collada_init})
modifier = type('modifier', (tag, Node), {'name': 'modifier', '__init__': collada_init})
morph = type('morph', (tag, Node), {'name': 'morph', '__init__': collada_init})
motion = type('motion', (tag, Node), {'name': 'motion', '__init__': collada_init})
multisample_enable = type('multisample_enable', (tag, Node), {'name': 'multisample_enable', '__init__': collada_init})
Name_array = type('Name_array', (tag, Node), {'name': 'Name_array', '__init__': collada_init})
emantic = type('emantic', (tag, Node), {'name': 'emantic', '__init__': collada_init})
newparam = type('newparam', (tag, Node), {'name': 'newparam', '__init__': collada_init})
ommon = type('ommon', (tag, Node), {'name': 'ommon', '__init__': collada_init})
node = type('node', (tag, Node), {'name': 'node', '__init__': collada_init})
normalize_enable = type('normalize_enable', (tag, Node), {'name': 'normalize_enable', '__init__': collada_init})
nurbs_surface = type('nurbs_surface', (tag, Node), {'name': 'nurbs_surface', '__init__': collada_init})
nurbs = type('nurbs', (tag, Node), {'name': 'nurbs', '__init__': collada_init})
optics = type('optics', (tag, Node), {'name': 'optics', '__init__': collada_init})
orient = type('orient', (tag, Node), {'name': 'orient', '__init__': collada_init})
origin = type('origin', (tag, Node), {'name': 'origin', '__init__': collada_init})
orthographic = type('orthographic', (tag, Node), {'name': 'orthographic', '__init__': collada_init})
p = type('p', (tag, Node), {'name': 'p', '__init__': collada_init})
edges = type('edges', (tag, Node), {'name': 'edges', '__init__': collada_init})
faces = type('faces', (tag, Node), {'name': 'faces', '__init__': collada_init})
lines = type('lines', (tag, Node), {'name': 'lines', '__init__': collada_init})
pcurves = type('pcurves', (tag, Node), {'name': 'pcurves', '__init__': collada_init})
ph = type('ph', (tag, Node), {'name': 'ph', '__init__': collada_init})
solids = type('solids', (tag, Node), {'name': 'solids', '__init__': collada_init})
triangles = type('triangles', (tag, Node), {'name': 'triangles', '__init__': collada_init})
trifans = type('trifans', (tag, Node), {'name': 'trifans', '__init__': collada_init})
tristrips = type('tristrips', (tag, Node), {'name': 'tristrips', '__init__': collada_init})
wires = type('wires', (tag, Node), {'name': 'wires', '__init__': collada_init})
parabola = type('parabola', (tag, Node), {'name': 'parabola', '__init__': collada_init})
param = type('param', (tag, Node), {'name': 'param', '__init__': collada_init})
ph = type('ph', (tag, Node), {'name': 'ph', '__init__': collada_init})
phong = type('phong', (tag, Node), {'name': 'phong', '__init__': collada_init})
physics_material = type('physics_material', (tag, Node), {'name': 'physics_material', '__init__': collada_init})
physics_model = type('physics_model', (tag, Node), {'name': 'physics_model', '__init__': collada_init})
physics_scene = type('physics_scene', (tag, Node), {'name': 'physics_scene', '__init__': collada_init})
plane = type('plane', (tag, Node), {'name': 'plane', '__init__': collada_init})
point_distance_attenuation = type('point_distance_attenuation', (tag, Node), {'name': 'point_distance_attenuation', '__init__': collada_init})
point_fade_threshold_size = type('point_fade_threshold_size', (tag, Node), {'name': 'point_fade_threshold_size', '__init__': collada_init})
point_size_max = type('point_size_max', (tag, Node), {'name': 'point_size_max', '__init__': collada_init})
point_size_min = type('point_size_min', (tag, Node), {'name': 'point_size_min', '__init__': collada_init})
point_size = type('point_size', (tag, Node), {'name': 'point_size', '__init__': collada_init})
point_smooth_enable = type('point_smooth_enable', (tag, Node), {'name': 'point_smooth_enable', '__init__': collada_init})
polygon_mode = type('polygon_mode', (tag, Node), {'name': 'polygon_mode', '__init__': collada_init})
polygon_offset_fill_enable = type('polygon_offset_fill_enable', (tag, Node), {'name': 'polygon_offset_fill_enable', '__init__': collada_init})
polygon_offset_line_enable = type('polygon_offset_line_enable', (tag, Node), {'name': 'polygon_offset_line_enable', '__init__': collada_init})
polygon_offset_point_enable = type('polygon_offset_point_enable', (tag, Node), {'name': 'polygon_offset_point_enable', '__init__': collada_init})
polygon_offset = type('polygon_offset', (tag, Node), {'name': 'polygon_offset', '__init__': collada_init})
polygon_smooth_enable = type('polygon_smooth_enable', (tag, Node), {'name': 'polygon_smooth_enable', '__init__': collada_init})
polygon_stipple_enable = type('polygon_stipple_enable', (tag, Node), {'name': 'polygon_stipple_enable', '__init__': collada_init})
polygons = type('polygons', (tag, Node), {'name': 'polygons', '__init__': collada_init})
polylist = type('polylist', (tag, Node), {'name': 'polylist', '__init__': collada_init})
prismatic = type('prismatic', (tag, Node), {'name': 'prismatic', '__init__': collada_init})
profile_BRIDGE = type('profile_BRIDGE', (tag, Node), {'name': 'profile_BRIDGE', '__init__': collada_init})
profile_CG = type('profile_CG', (tag, Node), {'name': 'profile_CG', '__init__': collada_init})
profile_COMMON = type('profile_COMMON', (tag, Node), {'name': 'profile_COMMON', '__init__': collada_init})
verview = type('verview', (tag, Node), {'name': 'verview', '__init__': collada_init})
profile_GLES = type('profile_GLES', (tag, Node), {'name': 'profile_GLES', '__init__': collada_init})
profile_GLES2 = type('profile_GLES2', (tag, Node), {'name': 'profile_GLES2', '__init__': collada_init})
profile_GLSL = type('profile_GLSL', (tag, Node), {'name': 'profile_GLSL', '__init__': collada_init})
program = type('program', (tag, Node), {'name': 'program', '__init__': collada_init})
projection_matrix = type('projection_matrix', (tag, Node), {'name': 'projection_matrix', '__init__': collada_init})
quadratic_attenuation = type('quadratic_attenuation', (tag, Node), {'name': 'quadratic_attenuation', '__init__': collada_init})
point = type('point', (tag, Node), {'name': 'point', '__init__': collada_init})
radius = type('radius', (tag, Node), {'name': 'radius', '__init__': collada_init})
capsule = type('capsule', (tag, Node), {'name': 'capsule', '__init__': collada_init})
circle = type('circle', (tag, Node), {'name': 'circle', '__init__': collada_init})
cone = type('cone', (tag, Node), {'name': 'cone', '__init__': collada_init})
cylinder = type('cylinder', (tag, Node), {'name': 'cylinder', '__init__': collada_init})
ellipse = type('ellipse', (tag, Node), {'name': 'ellipse', '__init__': collada_init})
hyperbola = type('hyperbola', (tag, Node), {'name': 'hyperbola', '__init__': collada_init})
torus = type('torus', (tag, Node), {'name': 'torus', '__init__': collada_init})
ref_attachment = type('ref_attachment', (tag, Node), {'name': 'ref_attachment', '__init__': collada_init})
ref = type('ref', (tag, Node), {'name': 'ref', '__init__': collada_init})
binary = type('binary', (tag, Node), {'name': 'binary', '__init__': collada_init})
init_from = type('init_from', (tag, Node), {'name': 'init_from', '__init__': collada_init})
reflective = type('reflective', (tag, Node), {'name': 'reflective', '__init__': collada_init})
reflectivity = type('reflectivity', (tag, Node), {'name': 'reflectivity', '__init__': collada_init})
render = type('render', (tag, Node), {'name': 'render', '__init__': collada_init})
renderable = type('renderable', (tag, Node), {'name': 'renderable', '__init__': collada_init})
rescale_normal_enable = type('rescale_normal_enable', (tag, Node), {'name': 'rescale_normal_enable', '__init__': collada_init})
restitution = type('restitution', (tag, Node), {'name': 'restitution', '__init__': collada_init})
revision = type('revision', (tag, Node), {'name': 'revision', '__init__': collada_init})
revolute = type('revolute', (tag, Node), {'name': 'revolute', '__init__': collada_init})
RGB = type('RGB', (tag, Node), {'name': 'RGB', '__init__': collada_init})
rigid_constraint = type('rigid_constraint', (tag, Node), {'name': 'rigid_constraint', '__init__': collada_init})
rotate = type('rotate', (tag, Node), {'name': 'rotate', '__init__': collada_init})
sample_alpha_to_coverage_enable = type('sample_alpha_to_coverage_enable', (tag, Node), {'name': 'sample_alpha_to_coverage_enable', '__init__': collada_init})
sample_alpha_to_one_enable = type('sample_alpha_to_one_enable', (tag, Node), {'name': 'sample_alpha_to_one_enable', '__init__': collada_init})
sample_coverage_enable = type('sample_coverage_enable', (tag, Node), {'name': 'sample_coverage_enable', '__init__': collada_init})
sample_coverage = type('sample_coverage', (tag, Node), {'name': 'sample_coverage', '__init__': collada_init})
sampler_image = type('sampler_image', (tag, Node), {'name': 'sampler_image', '__init__': collada_init})
sampler_states = type('sampler_states', (tag, Node), {'name': 'sampler_states', '__init__': collada_init})
sampler = type('sampler', (tag, Node), {'name': 'sampler', '__init__': collada_init})
nterpolation = type('nterpolation', (tag, Node), {'name': 'nterpolation', '__init__': collada_init})
sampler1D = type('sampler1D', (tag, Node), {'name': 'sampler1D', '__init__': collada_init})
sampler2D = type('sampler2D', (tag, Node), {'name': 'sampler2D', '__init__': collada_init})
sampler3D = type('sampler3D', (tag, Node), {'name': 'sampler3D', '__init__': collada_init})
samplerCUBE = type('samplerCUBE', (tag, Node), {'name': 'samplerCUBE', '__init__': collada_init})
samplerDEPTH = type('samplerDEPTH', (tag, Node), {'name': 'samplerDEPTH', '__init__': collada_init})
samplerRECT = type('samplerRECT', (tag, Node), {'name': 'samplerRECT', '__init__': collada_init})
scale = type('scale', (tag, Node), {'name': 'scale', '__init__': collada_init})
scene = type('scene', (tag, Node), {'name': 'scene', '__init__': collada_init})
scissor_test_enable = type('scissor_test_enable', (tag, Node), {'name': 'scissor_test_enable', '__init__': collada_init})
scissor = type('scissor', (tag, Node), {'name': 'scissor', '__init__': collada_init})
semantic = type('semantic', (tag, Node), {'name': 'semantic', '__init__': collada_init})
setparam = type('setparam', (tag, Node), {'name': 'setparam', '__init__': collada_init})
shade_model = type('shade_model', (tag, Node), {'name': 'shade_model', '__init__': collada_init})
shader = type('shader', (tag, Node), {'name': 'shader', '__init__': collada_init})
shape = type('shape', (tag, Node), {'name': 'shape', '__init__': collada_init})
shells = type('shells', (tag, Node), {'name': 'shells', '__init__': collada_init})
shininess = type('shininess', (tag, Node), {'name': 'shininess', '__init__': collada_init})
SIDREF_array = type('SIDREF_array', (tag, Node), {'name': 'SIDREF_array', '__init__': collada_init})
size_exact = type('size_exact', (tag, Node), {'name': 'size_exact', '__init__': collada_init})
size_ratio = type('size_ratio', (tag, Node), {'name': 'size_ratio', '__init__': collada_init})
size = type('size', (tag, Node), {'name': 'size', '__init__': collada_init})
create_cube = type('create_cube', (tag, Node), {'name': 'create_cube', '__init__': collada_init})
create3d = type('create3d', (tag, Node), {'name': 'create3d', '__init__': collada_init})
skeleton = type('skeleton', (tag, Node), {'name': 'skeleton', '__init__': collada_init})
skew = type('skew', (tag, Node), {'name': 'skew', '__init__': collada_init})
skin = type('skin', (tag, Node), {'name': 'skin', '__init__': collada_init})
source_data = type('source_data', (tag, Node), {'name': 'source_data', '__init__': collada_init})
source = type('source', (tag, Node), {'name': 'source', '__init__': collada_init})
sources = type('sources', (tag, Node), {'name': 'sources', '__init__': collada_init})
specular = type('specular', (tag, Node), {'name': 'specular', '__init__': collada_init})
speed = type('speed', (tag, Node), {'name': 'speed', '__init__': collada_init})
sphere = type('sphere', (tag, Node), {'name': 'sphere', '__init__': collada_init})
spline = type('spline', (tag, Node), {'name': 'spline', '__init__': collada_init})
nterpolation = type('nterpolation', (tag, Node), {'name': 'nterpolation', '__init__': collada_init})
spot = type('spot', (tag, Node), {'name': 'spot', '__init__': collada_init})
spring = type('spring', (tag, Node), {'name': 'spring', '__init__': collada_init})
states = type('states', (tag, Node), {'name': 'states', '__init__': collada_init})
static_friction = type('static_friction', (tag, Node), {'name': 'static_friction', '__init__': collada_init})
stencil_clear = type('stencil_clear', (tag, Node), {'name': 'stencil_clear', '__init__': collada_init})
stencil_func_separate = type('stencil_func_separate', (tag, Node), {'name': 'stencil_func_separate', '__init__': collada_init})
stencil_func = type('stencil_func', (tag, Node), {'name': 'stencil_func', '__init__': collada_init})
stencil_mask_separate = type('stencil_mask_separate', (tag, Node), {'name': 'stencil_mask_separate', '__init__': collada_init})
stencil_mask = type('stencil_mask', (tag, Node), {'name': 'stencil_mask', '__init__': collada_init})
stencil_op_separate = type('stencil_op_separate', (tag, Node), {'name': 'stencil_op_separate', '__init__': collada_init})
stencil_op = type('stencil_op', (tag, Node), {'name': 'stencil_op', '__init__': collada_init})
stencil_target = type('stencil_target', (tag, Node), {'name': 'stencil_target', '__init__': collada_init})
stencil_test_enable = type('stencil_test_enable', (tag, Node), {'name': 'stencil_test_enable', '__init__': collada_init})
stiffness = type('stiffness', (tag, Node), {'name': 'stiffness', '__init__': collada_init})
subject = type('subject', (tag, Node), {'name': 'subject', '__init__': collada_init})
surface_curves = type('surface_curves', (tag, Node), {'name': 'surface_curves', '__init__': collada_init})
surface = type('surface', (tag, Node), {'name': 'surface', '__init__': collada_init})
surfaces = type('surfaces', (tag, Node), {'name': 'surfaces', '__init__': collada_init})
swept_surface = type('swept_surface', (tag, Node), {'name': 'swept_surface', '__init__': collada_init})
swing_cone_and_twist = type('swing_cone_and_twist', (tag, Node), {'name': 'swing_cone_and_twist', '__init__': collada_init})
target_value = type('target_value', (tag, Node), {'name': 'target_value', '__init__': collada_init})
target = type('target', (tag, Node), {'name': 'target', '__init__': collada_init})
targets = type('targets', (tag, Node), {'name': 'targets', '__init__': collada_init})
bind_material = type('bind_material', (tag, Node), {'name': 'bind_material', '__init__': collada_init})
formula = type('formula', (tag, Node), {'name': 'formula', '__init__': collada_init})
instance_rigid_body = type('instance_rigid_body', (tag, Node), {'name': 'instance_rigid_body', '__init__': collada_init})
kinematics_model = type('kinematics_model', (tag, Node), {'name': 'kinematics_model', '__init__': collada_init})
kinematics = type('kinematics', (tag, Node), {'name': 'kinematics', '__init__': collada_init})
light = type('light', (tag, Node), {'name': 'light', '__init__': collada_init})
motion = type('motion', (tag, Node), {'name': 'motion', '__init__': collada_init})
optics = type('optics', (tag, Node), {'name': 'optics', '__init__': collada_init})
rigid_body = type('rigid_body', (tag, Node), {'name': 'rigid_body', '__init__': collada_init})
verview = type('verview', (tag, Node), {'name': 'verview', '__init__': collada_init})
technique_hint = type('technique_hint', (tag, Node), {'name': 'technique_hint', '__init__': collada_init})
technique_override = type('technique_override', (tag, Node), {'name': 'technique_override', '__init__': collada_init})
texcombiner = type('texcombiner', (tag, Node), {'name': 'texcombiner', '__init__': collada_init})
texcoord = type('texcoord', (tag, Node), {'name': 'texcoord', '__init__': collada_init})
texenv = type('texenv', (tag, Node), {'name': 'texenv', '__init__': collada_init})
texture_env_color = type('texture_env_color', (tag, Node), {'name': 'texture_env_color', '__init__': collada_init})
texture_env_mode = type('texture_env_mode', (tag, Node), {'name': 'texture_env_mode', '__init__': collada_init})
texture_pipeline = type('texture_pipeline', (tag, Node), {'name': 'texture_pipeline', '__init__': collada_init})
texture = type('texture', (tag, Node), {'name': 'texture', '__init__': collada_init})
texture1D_enable = type('texture1D_enable', (tag, Node), {'name': 'texture1D_enable', '__init__': collada_init})
texture1D = type('texture1D', (tag, Node), {'name': 'texture1D', '__init__': collada_init})
texture2D_enable = type('texture2D_enable', (tag, Node), {'name': 'texture2D_enable', '__init__': collada_init})
texture2D = type('texture2D', (tag, Node), {'name': 'texture2D', '__init__': collada_init})
texture3D_enable = type('texture3D_enable', (tag, Node), {'name': 'texture3D_enable', '__init__': collada_init})
texture3D = type('texture3D', (tag, Node), {'name': 'texture3D', '__init__': collada_init})
textureCUBE_enable = type('textureCUBE_enable', (tag, Node), {'name': 'textureCUBE_enable', '__init__': collada_init})
textureCUBE = type('textureCUBE', (tag, Node), {'name': 'textureCUBE', '__init__': collada_init})
textureDEPTH_enable = type('textureDEPTH_enable', (tag, Node), {'name': 'textureDEPTH_enable', '__init__': collada_init})
textureDEPTH = type('textureDEPTH', (tag, Node), {'name': 'textureDEPTH', '__init__': collada_init})
textureRECT_enable = type('textureRECT_enable', (tag, Node), {'name': 'textureRECT_enable', '__init__': collada_init})
textureRECT = type('textureRECT', (tag, Node), {'name': 'textureRECT', '__init__': collada_init})
time_step = type('time_step', (tag, Node), {'name': 'time_step', '__init__': collada_init})
title = type('title', (tag, Node), {'name': 'title', '__init__': collada_init})
translate = type('translate', (tag, Node), {'name': 'translate', '__init__': collada_init})
transparency = type('transparency', (tag, Node), {'name': 'transparency', '__init__': collada_init})
transparent = type('transparent', (tag, Node), {'name': 'transparent', '__init__': collada_init})
unit = type('unit', (tag, Node), {'name': 'unit', '__init__': collada_init})
unnormalized = type('unnormalized', (tag, Node), {'name': 'unnormalized', '__init__': collada_init})
usertype = type('usertype', (tag, Node), {'name': 'usertype', '__init__': collada_init})
v = type('v', (tag, Node), {'name': 'v', '__init__': collada_init})
value = type('value', (tag, Node), {'name': 'value', '__init__': collada_init})
velocity = type('velocity', (tag, Node), {'name': 'velocity', '__init__': collada_init})
vertex_weights = type('vertex_weights', (tag, Node), {'name': 'vertex_weights', '__init__': collada_init})
wrap_p = type('wrap_p', (tag, Node), {'name': 'wrap_p', '__init__': collada_init})
wrap_s = type('wrap_s', (tag, Node), {'name': 'wrap_s', '__init__': collada_init})
wrap_t = type('wrap_t', (tag, Node), {'name': 'wrap_t', '__init__': collada_init})
xfov = type('xfov', (tag, Node), {'name': 'xfov', '__init__': collada_init})
xmag = type('xmag', (tag, Node), {'name': 'xmag', '__init__': collada_init})
yfov = type('yfov', (tag, Node), {'name': 'yfov', '__init__': collada_init})
ymag = type('ymag', (tag, Node), {'name': 'ymag', '__init__': collada_init})
zfar = type('zfar', (tag, Node), {'name': 'zfar', '__init__': collada_init})
znear = type('znear', (tag, Node), {'name': 'znear', '__init__': collada_init})

# builtins require an underscore.
_input = type('input', (closed_tag, Node), {'name': 'input', '__init__': collada_init})
_float = type('float', (tag, Node), {'name': 'float', '__init__': collada_init})
_pass = type('pass', (tag, Node), {'name': 'pass', '__init__': collada_init})
_max = type('max', (tag, Node), {'name': 'max', '__init__': collada_init})
_min = type('min', (tag, Node), {'name': 'min', '__init__': collada_init})
_import = type('import', (tag, Node), {'name': 'import', '__init__': collada_init})
_hex = type('hex', (tag, Node), {'name': 'hex', '__init__': collada_init})
_format = type('format', (tag, Node), {'name': 'format', '__init__': collada_init})


class xml_header():
    """xml

    Returns:
        str: <?xml version="1.0"?>
    """
    def __str__(self):
        return '<?xml version="1.0"?>'


'''

# The asset tag describes the author and environment of the file
TMP_ASSET = asset(
    author("alorino"),
    up_axis("Y_UP")
)

# common cameras are perspective or orthographic
LIB_CAMERAS = library_cameras(
    camera(_id="PerspCamera", _name="PerspCamera").appendChild(
        optics(
            technique_common(
                perspective(
                    yfov(37.8493),
                    aspect_ratio(1),
                    znear(10),
                    zfar(1000)
                )
            )
        )
    )
)

# Common lights are point, spot or directional
LIB_LIGHTS = library_lights(
    light(_id="pointLightShape1-lib", _name="pointLightShape1").appendChild(
        technique_common(
            point(
                color("1 1 1"),  # need to figure this out
                constant_attenuation(1),
                linear_attenuation(0),
                quadratic_attenuation(0)
            )
        )
    )
)

# Materials instance effects
LIB_MAT = library_materials(
    material(_id="Blue", _name="Blue").appendChild(
        instance_effect(_url="#Blue-fx")
    )
)

# Common effects act like the OpenGL 1 state
LIB_FX = library_effects(
    effect(_id="Blue-fx").appendChild(
        profile_COMMON(
            technique(_sid="common").appendChild(
                phong(
                    emission(
                        color("0 0 0 1"),
                        index_of_refraction(
                            _float(0)
                        )
                    )
                )
            )
        )
    )
)

# Geometry describes the OpenGL attributes
LIB_GEOM = library_geometries(
    geometry(_id="box-lib", _name="box").appendChild(
        mesh(
            source(_id="box-lib-positions", _name="position"),
            source(_id="box-lib-normals", _name="normal"),
            vertices(_id="box-lib-vertices").appendChild(
                _input(_semantic="POSITION", _source="#box-lib-positions")
            ),
            polylist(_count="6", _material="BlueSG").appendChild(
                _input(_offset="0", _semantic="VERTEX", _source="#box-lib-vertices"),
                _input(_offset="1", _semantic="NORMAL", _source="#box-lib-normals"),
                vcount("4 4 4 4 4 4"),
                p("0 0 2 1 3 2 1 3 0 4 1 5 5 6 4 7")
            )
        )
    )
)

# The scene tag specifies a visual scene and sometimes a phyiscs scene
LIB_SCENES = library_visual_scenes(
    visual_scene(_id="VisualSceneNode", _name="untitled").appendChild(
        node(_id="Camera", _name="Camera")
    )
)

TMP_SCENE = scene(
    instance_visual_scene(_url="#VisualSceneNode")
)

# Extras are a way of extending the format
TMP_EXTRA = extra(
    technique(_profile="steveT").appendChild(
        test_element(_id="my_test_element", _attr1="value1", _attr2="value2").appendChild(
            "this is some text"
        )
    )
)

'''

"""

library_cameras(
    camera(_id="Camera-camera", _name="Camera").html(
        optics(
            technique_common(
                perspective(
                    xfov("49.13434", _sid="xfov")
                    aspect_ratio("1.777778")  # NOTE - underscored attributes?. - likely to break?
                    znear("0.1", _sid="znear")
                    zfar("100", _sid="zfar")
                )
            )
        )
    )
)

source(_id="Cube-mesh-normals").html(
    float_array(_id="Cube-mesh-normals-array", _count="18").html("0 0 -1 0 0 1 1 -2.83e-7 0 -2.83e-7 -1 0 -1 2.23e-7 -1.34e-7 2.38e-7 1 2.08e-7"),
        technique_common(
            accessor(_source="#Cube-mesh-normals-array", _count="6", _stride="3").html(
                param(_name="X" type="float"),
                param(_name="Y" type="float"),
            param(_name="Z" type="float")
        )
    )
)

vertices(_id="Cube-mesh-vertices").html(
    input(_semantic="POSITION", _source="#Cube-mesh-positions")
)


COLLADA(
    asset(
        created("2005-06-27T21:00:00Z"),
        keywords("COLLADA inthange"),
        created("2005-06-27T21:00:00Z"),
        unit(_name="nautical_league", _meter="5556.0"),
        up_axis("Z_UP")
    )
)

light(_id="blue").html(
    technique_common(
        directional(
        color(0.1 0.1 0.5)
        )
    )
)


camera(_name="eyepoint").html(
    optics(
        technique_common("....")
        technique profile="MyFancyGIRenderer">
            param("180.0", _name="FocalLength", _type="float")
            param("5.6", _name="Aperture", _type="float")
        )
    ),
    imager(
        technique(_profile="MyFancyGIRenderer").html(
            param("200.0", _name="ShutterSpeed", _type="float")
            param("0.2", _name="RedGain", _type="float")
            param("0.22", _name="GreenGain", _type="float")
            param("0.25", _name="BlueGain", _type="float")
            param("2.2", _name="RedGamma", _type="float")
            param("2.1", _name="GreenGamma", _type="float")
            param("2.17", _name="BlueGamma", _type="float")
            param("0.17", _name="BloomPixelLeak", _type="float")
            param("InvSquare", _name="BloomFalloff", _type="Name")
        )
    )
)


+ asset
+ + contributor
+ + + author
+ + + authoring_tool
+ + + created
+ + + unit
+ + + up_axi
+ library_cameras
+ + camera
+ + + optics
+ + + + technique_common
+ + + + + perspective
+ + + + + + xfov
+ + + + + + aspect_ratio
+ + + + + + znear
+ + + + + + zfar
+ library_lights
+ + light
+ + + technique_common
+ + + + point
+ + + + + color
+ + + + + constant_attenuation
+ + + + + linear_attenuation
+ + + + + quadratic_attenuation
+ + + extra
+ + + + technique
............
+ library_visual_scenes
+ + visual_scene
+ + + node
+ + + + translate
+ + + + rotate
+ + + + rotate
+ + + + rotate
+ + + + scale
+ + + + instance_geometry
+ + + + + bind_material
+ + + + + + technique_common
+ + + + + + + instance_material
+ + + node
+ + + + translate
+ + + + rotate
+ + + + rotate
+ + + + rotate
+ + + + scale
+ + + + instance_camera
+ scene
+ + instance_visual_scene

"""



"""
    test_domonic
    ~~~~~~~~~~~~
    unit tests for css
"""

import unittest
import json
# import requests
# from mock import patch

from domonic.html import *
from domonic.JSON import *


# class TestCase(unittest.TestCase):

# if __name__ == '__main__':
#     unittest.main()
